@@ -108,7 +108,7 @@ import java.util.Vector;
  <!-- options-end -->
  *
  * @author Mark Hall (mhall@cs.waikato.ac.nz)
- * @version $Revision: 1.31 $
+ * @version $Revision: 7433 $
  */
 public class WrapperSubsetEval
   extends ASEvaluation
@@ -478,10 +478,12 @@ public class WrapperSubsetEval
   public Capabilities getCapabilities() {
     Capabilities	result;
     
-    if (getClassifier() == null)
+    if (getClassifier() == null) {
       result = super.getCapabilities();
-    else
+      result.disableAll();
+    } else {
       result = getClassifier().getCapabilities();
+    }
     
     // set dependencies
     for (Capability cap: Capability.values())
@@ -633,6 +635,12 @@ public class WrapperSubsetEval
     int i;
     double mean = 0;
     double variance = 0;
+    
+    // setting a threshold less than zero allows for "manual" exploration
+    // and prevents multiple xval for each subset
+    if (m_threshold < 0) { 
+      return false; 
+    }
 
     if (entries == 1) {
       return  true;
@@ -667,7 +675,7 @@ public class WrapperSubsetEval
    * @return		the revision
    */
   public String getRevision() {
-    return RevisionUtils.extract("$Revision: 1.31 $");
+    return RevisionUtils.extract("$Revision: 7433 $");
   }
 
   /**
