@@ -127,6 +127,14 @@ import java.util.Vector;
  * <pre> -Z
  *  Turns on normalization of input data (default: off)</pre>
  * 
+ * <pre> -J
+ *  Turn off nominal to binary conversion.
+ *  WARNING: use only if your data is all numeric!</pre>
+ * 
+ * <pre> -V
+ *  Turn off missing value replacement.
+ *  WARNING: use only if your data has no missing values.</pre>
+ * 
  * <pre> -P &lt;double&gt;
  *  Set the epsilon in loss function of epsilon-SVR (default: 0.1)</pre>
  * 
@@ -141,7 +149,9 @@ import java.util.Vector;
  * 
  * <pre> -W &lt;double&gt;
  *  Set the parameters C of class i to weight[i]*C, for C-SVC
- *   (default: 1)</pre>
+ *  E.g., for a 3-class problem, you could use "1 1 1" for equally
+ *  weighted classes.
+ *  (default: 1 for all classes)</pre>
  * 
  * <pre> -B
  *  Trains a SVC model instead of a SVR one (default: SVR)</pre>
@@ -154,7 +164,7 @@ import java.util.Vector;
  *
  * @author  Yasser EL-Manzalawy
  * @author  FracPete (fracpete at waikato dot ac dot nz)
- * @version $Revision: 4852 $
+ * @version $Revision: 7561 $
  * @see     weka.core.converters.LibSVMLoader
  * @see     weka.core.converters.LibSVMSaver
  */
@@ -432,7 +442,9 @@ public class LibSVM
     result.addElement(
         new Option(
             "\tSet the parameters C of class i to weight[i]*C, for C-SVC\n" 
-            + "\t (default: 1)",
+            + "\tE.g., for a 3-class problem, you could use \"1 1 1\" for equally\n"
+            + "\tweighted classes.\n"
+            + "\t(default: 1 for all classes)",
             "W", 1, "-W <double>"));
     
     result.addElement(
@@ -488,6 +500,14 @@ public class LibSVM
    * <pre> -Z
    *  Turns on normalization of input data (default: off)</pre>
    * 
+   * <pre> -J
+   *  Turn off nominal to binary conversion.
+   *  WARNING: use only if your data is all numeric!</pre>
+   * 
+   * <pre> -V
+   *  Turn off missing value replacement.
+   *  WARNING: use only if your data has no missing values.</pre>
+   * 
    * <pre> -P &lt;double&gt;
    *  Set the epsilon in loss function of epsilon-SVR (default: 0.1)</pre>
    * 
@@ -502,7 +522,9 @@ public class LibSVM
    * 
    * <pre> -W &lt;double&gt;
    *  Set the parameters C of class i to weight[i]*C, for C-SVC
-   *   (default: 1)</pre>
+   *  E.g., for a 3-class problem, you could use "1 1 1" for equally
+   *  weighted classes.
+   *  (default: 1 for all classes)</pre>
    * 
    * <pre> -B
    *  Trains a SVC model instead of a SVR one (default: SVR)</pre>
@@ -1087,7 +1109,7 @@ public class LibSVM
    *         displaying in the explorer/experimenter gui
    */
   public String weightsTipText() {
-    return "The weights to use for the classes, if empty 1 is used by default.";
+    return "The weights to use for the classes (blank-separated list, eg, \"1 1 1\" for a 3-class problem), if empty 1 is used by default.";
   }
   
   /**
@@ -1452,7 +1474,9 @@ public class LibSVM
 	  if (v > 0)
 	    result[0] = 1;
 	  else
-	    result[0] = Double.NaN;  // outlier
+	    // outlier (interface for Classifier specifies that unclassified instances                                       
+            // should return a distribution of all zeros)
+	    result[0] = 0;
 	}
 	else {
 	  result[(int) v] = 1;
@@ -1473,6 +1497,7 @@ public class LibSVM
    */
   public Capabilities getCapabilities() {
     Capabilities result = super.getCapabilities();
+    result.disableAll();
 
     // attributes
     result.enable(Capability.NOMINAL_ATTRIBUTES);
@@ -1606,7 +1631,7 @@ public class LibSVM
    * @return		the revision
    */
   public String getRevision() {
-    return RevisionUtils.extract("$Revision: 4852 $");
+    return RevisionUtils.extract("$Revision: 7561 $");
   }
   
   /**
