@@ -32,6 +32,7 @@ import weka.core.Instance;
 import weka.core.Instances;
 import weka.core.Option;
 import weka.core.OptionHandler;
+import weka.core.Randomizable;
 import weka.core.RevisionHandler;
 import weka.core.RevisionUtils;
 import weka.core.Utils;
@@ -142,15 +143,15 @@ import javax.swing.JTextField;
  <!-- options-end -->
  *
  * @author Malcolm Ware (mfw4@cs.waikato.ac.nz)
- * @version $Revision: 1.12 $
+ * @version $Revision: 6563 $
  */
 public class MultilayerPerceptron 
   extends Classifier 
-  implements OptionHandler, WeightedInstancesHandler {
+  implements OptionHandler, WeightedInstancesHandler, Randomizable {
   
   /** for serialization */
-  static final long serialVersionUID = 572250905027665169L;
-  
+  private static final long serialVersionUID = -5990607817048210779L;
+
   /**
    * Main method for testing this class.
    *
@@ -373,6 +374,26 @@ public class MultilayerPerceptron
       }
     }
     
+    /**
+     * Call this to have the connection save the current
+     * weights.
+     */
+    public void saveWeights() {
+      for (int i = 0; i < m_numInputs; i++) {
+        m_inputList[i].saveWeights();
+      }
+    }
+    
+    /**
+     * Call this to have the connection restore from the saved
+     * weights.
+     */
+    public void restoreWeights() {
+      for (int i = 0; i < m_numInputs; i++) {
+        m_inputList[i].restoreWeights();
+      }
+    }
+    
     
     /** 
      * Call this function to set What this end unit represents.
@@ -413,7 +434,7 @@ public class MultilayerPerceptron
      * @return		the revision
      */
     public String getRevision() {
-      return RevisionUtils.extract("$Revision: 1.12 $");
+      return RevisionUtils.extract("$Revision: 6563 $");
     }
   }
   
@@ -664,14 +685,14 @@ public class MultilayerPerceptron
      * @return		the revision
      */
     public String getRevision() {
-      return RevisionUtils.extract("$Revision: 1.12 $");
+      return RevisionUtils.extract("$Revision: 6563 $");
     }
   }
 
   /** 
    * This provides the basic controls for working with the neuralnetwork
    * @author Malcolm Ware (mfw4@cs.waikato.ac.nz)
-   * @version $Revision: 1.12 $
+   * @version $Revision: 6563 $
    */
   class ControlPanel 
     extends JPanel
@@ -867,13 +888,16 @@ public class MultilayerPerceptron
      * @return		the revision
      */
     public String getRevision() {
-      return RevisionUtils.extract("$Revision: 1.12 $");
+      return RevisionUtils.extract("$Revision: 6563 $");
     }
   }
   
-
-  /** a ZeroR model in case no model can be built from the data */
+  /** a ZeroR model in case no model can be built from the data 
+   * or the network predicts all zeros for the classes */
   private Classifier m_ZeroR;
+
+  /** Whether to use the default ZeroR model */
+  private boolean m_useDefaultModel = false;
     
   /** The training instances. */
   private Instances m_instances;
@@ -949,7 +973,7 @@ public class MultilayerPerceptron
   private int m_driftThreshold;
 
   /** The number used to seed the random number generator. */
-  private long m_randomSeed;
+  private int m_randomSeed;
 
   /** The actual random number generator. */
   private Random m_random;
@@ -1144,7 +1168,7 @@ public class MultilayerPerceptron
    * number is needed for the network.
    * @param l The seed.
    */
-  public void setRandomSeed(long l) {
+  public void setSeed(int l) {
     if (l >= 0) {
       m_randomSeed = l;
     }
@@ -1153,7 +1177,7 @@ public class MultilayerPerceptron
   /**
    * @return The seed for the random number generator.
    */
-  public long getRandomSeed() {
+  public int getSeed() {
     return m_randomSeed;
   }
 
@@ -1707,6 +1731,7 @@ public class MultilayerPerceptron
    */
   public Capabilities getCapabilities() {
     Capabilities result = super.getCapabilities();
+    result.disableAll();
 
     // attributes
     result.enable(Capability.NOMINAL_ATTRIBUTES);
@@ -1738,17 +1763,18 @@ public class MultilayerPerceptron
     i = new Instances(i);
     i.deleteWithMissingClass();
 
-    // only class? -> build ZeroR model
+    m_ZeroR = new weka.classifiers.rules.ZeroR();
+    m_ZeroR.buildClassifier(i);
+    // only class? -> use ZeroR model
     if (i.numAttributes() == 1) {
       System.err.println(
 	  "Cannot build model (only class attribute present in data!), "
 	  + "using ZeroR model instead!");
-      m_ZeroR = new weka.classifiers.rules.ZeroR();
-      m_ZeroR.buildClassifier(i);
+      m_useDefaultModel = true;
       return;
     }
     else {
-      m_ZeroR = null;
+      m_useDefaultModel = false;
     }
     
     m_epoch = 0;
@@ -1884,6 +1910,7 @@ public class MultilayerPerceptron
     double right = 0;
     double driftOff = 0;
     double lastRight = Double.POSITIVE_INFINITY;
+    double bestError = Double.POSITIVE_INFINITY;
     double tempRate;
     double totalWeight = 0;
     double totalValWeight = 0;
@@ -1974,13 +2001,24 @@ public class MultilayerPerceptron
 	}
 	
 	if (right < lastRight) {
-	  driftOff = 0;
+	  
+	  if (right < bestError) {
+	    bestError = right;
+	    // save the network weights at this point
+	    for (int noc = 0; noc < m_numClasses; noc++) {
+	      m_outputs[noc].saveWeights();
+	    }
+	    driftOff = 0;
+	  }
 	}
 	else {
 	  driftOff++;
 	}
 	lastRight = right;
 	if (driftOff > m_driftThreshold || noa + 1 >= m_numEpochs) {
+	  for (int noc = 0; noc < m_numClasses; noc++) {
+            m_outputs[noc].restoreWeights();
+          }
 	  m_accepted = true;
 	}
 	right /= totalValWeight;
@@ -2051,7 +2089,7 @@ public class MultilayerPerceptron
   public double[] distributionForInstance(Instance i) throws Exception {
 
     // default model?
-    if (m_ZeroR != null) {
+    if (m_useDefaultModel) {
       return m_ZeroR.distributionForInstance(i);
     }
     
@@ -2096,7 +2134,7 @@ public class MultilayerPerceptron
       count += theArray[noa];
     }
     if (count <= 0) {
-      return null;
+      return m_ZeroR.distributionForInstance(i);
     }
     for (int noa = 0; noa < m_numClasses; noa++) {
       theArray[noa] /= count;
@@ -2281,9 +2319,9 @@ public class MultilayerPerceptron
     }
     String seedString = Utils.getOption('S', options);
     if (seedString.length() != 0) {
-      setRandomSeed(Long.parseLong(seedString));
+      setSeed(Integer.parseInt(seedString));
     } else {
-      setRandomSeed(0);
+      setSeed(0);
     }
     String thresholdString = Utils.getOption('E', options);
     if (thresholdString.length() != 0) {
@@ -2351,7 +2389,7 @@ public class MultilayerPerceptron
     options[current++] = "-M"; options[current++] = "" + getMomentum();
     options[current++] = "-N"; options[current++] = "" + getTrainingTime(); 
     options[current++] = "-V"; options[current++] = "" +getValidationSetSize();
-    options[current++] = "-S"; options[current++] = "" + getRandomSeed();
+    options[current++] = "-S"; options[current++] = "" + getSeed();
     options[current++] = "-E"; options[current++] =""+getValidationThreshold();
     options[current++] = "-H"; options[current++] = getHiddenLayers();
     if (getGUI()) {
@@ -2388,7 +2426,7 @@ public class MultilayerPerceptron
    */
   public String toString() {
     // only ZeroR model?
-    if (m_ZeroR != null) {
+    if (m_useDefaultModel) {
       StringBuffer buf = new StringBuffer();
       buf.append(this.getClass().getName().replaceAll(".*\\.", "") + "\n");
       buf.append(this.getClass().getName().replaceAll(".*\\.", "").replaceAll(".", "=") + "\n\n");
@@ -2490,7 +2528,7 @@ public class MultilayerPerceptron
   /**
    * @return a string to describe the random seed option.
    */
-  public String randomSeedTipText() {
+  public String seedTipText() {
     return "Seed used to initialise the random number generator." +
       "Random numbers are used for setting the initial weights of the" +
       " connections betweem nodes, and also for shuffling the training data.";
@@ -2650,6 +2688,6 @@ public class MultilayerPerceptron
    * @return		the revision
    */
   public String getRevision() {
-    return RevisionUtils.extract("$Revision: 1.12 $");
+    return RevisionUtils.extract("$Revision: 6563 $");
   }
 }
