@@ -34,7 +34,7 @@ import weka.core.TechnicalInformationHandler;
 import weka.core.TechnicalInformation.Field;
 import weka.core.TechnicalInformation.Type;
 
-/*
+/**
  <!-- globalinfo-start -->
  * Class for generating a multi-class alternating decision tree using the LogitBoost strategy. For more info, see<br/>
  * <br/>
@@ -71,7 +71,7 @@ import weka.core.TechnicalInformation.Type;
  <!-- options-end -->
  *
  * @author Richard Kirkby
- * @version $Revision: 4819 $
+ * @version $Revision: 6036 $
 */
 
 public class LADTree
@@ -1466,7 +1466,7 @@ public class LADTree
    * @return		the revision
    */
   public String getRevision() {
-    return RevisionUtils.extract("$Revision: 4819 $");
+    return RevisionUtils.extract("$Revision: 6036 $");
   }
 
   /**
@@ -1476,6 +1476,7 @@ public class LADTree
    */
   public Capabilities getCapabilities() {
     Capabilities result = super.getCapabilities();
+    result.disableAll();
 
     // attributes
     result.enable(Capability.NOMINAL_ATTRIBUTES);
