@@ -24,9 +24,11 @@ package weka.core;
 import weka.core.converters.ConverterUtils.DataSource;
 
 import java.io.Serializable;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Properties;
 import java.util.Vector;
 
@@ -62,7 +64,7 @@ import java.util.Vector;
  * </pre>
  * 
  * @author  FracPete (fracpete at waikato dot ac dot nz)
- * @version $Revision: 1.27 $
+ * @version $Revision: 7082 $
  */
 public class Capabilities 
   implements Cloneable, Serializable, RevisionHandler {
@@ -271,6 +273,11 @@ public class Capabilities
     m_MissingValuesTest          = Boolean.parseBoolean(PROPERTIES.getProperty("MissingValuesTest", "true")) && m_Test;
     m_MissingClassValuesTest     = Boolean.parseBoolean(PROPERTIES.getProperty("MissingClassValuesTest", "true")) && m_Test;
     m_MinimumNumberInstancesTest = Boolean.parseBoolean(PROPERTIES.getProperty("MinimumNumberInstancesTest", "true")) && m_Test;
+        
+    if (owner instanceof weka.classifiers.UpdateableClassifier ||
+        owner instanceof weka.clusterers.UpdateableClusterer) {
+      setMinimumNumberInstances(0);
+    }
   }
   
   /**
@@ -572,6 +579,18 @@ public class Capabilities
 	enableDependency(cap);
     }
   }
+  
+  /**
+   * enables all attribute and class types (including dependencies)
+   */
+  public void enableAll() {
+    enableAllAttributes();
+    enableAllAttributeDependencies();
+    enableAllClasses();
+    enableAllClassDependencies();
+    enable(Capability.MISSING_VALUES);
+    enable(Capability.MISSING_CLASS_VALUES);
+  }
 
   /**
    * disables the given capability
@@ -697,6 +716,19 @@ public class Capabilities
     }
   }
   
+  /**
+   * disables all attribute and class types (including dependencies)
+   */
+  public void disableAll() {
+    disableAllAttributes();
+    disableAllAttributeDependencies();
+    disableAllClasses();
+    disableAllClassDependencies();
+    disable(Capability.MISSING_VALUES);
+    disable(Capability.MISSING_CLASS_VALUES);
+    disable(Capability.NO_CLASS);
+  }
+  
   /**
    * returns all class capabilities
    * 
@@ -1338,13 +1370,45 @@ public class Capabilities
     // object name
     result.append(indentStr + capsName + " " + objectname + " = new " + capsName + "(this);\n");
     
+    List<Capability> capsList = new ArrayList<Capability>();
+    boolean hasNominalAtt = false;
+    boolean hasBinaryAtt = false;
+    boolean hasUnaryAtt = false;
+    boolean hasEmptyNomAtt = false;
+    boolean hasNominalClass = false;
     // capabilities
     result.append("\n");
     for (Capability cap: Capability.values()) {
       // capability
-      if (handles(cap))
-        result.append(
-            indentStr + objectname + ".enable(" + capName + "." + cap.name() + ");\n");
+      if (handles(cap)) {
+        if (cap == Capability.NOMINAL_ATTRIBUTES) {
+          hasNominalAtt = true;
+        }
+        if (cap == Capability.NOMINAL_CLASS) {
+          hasNominalClass = true;
+        }
+        if (cap == Capability.BINARY_ATTRIBUTES) {
+          hasBinaryAtt = true;
+        }
+        if (cap == Capability.UNARY_ATTRIBUTES) {
+          hasUnaryAtt = true;
+        }
+        if (cap == Capability.EMPTY_NOMINAL_ATTRIBUTES) {
+          hasEmptyNomAtt = true;
+        }
+        capsList.add(cap);              
+      }
+    }
+    
+    for (Capability cap : capsList) {
+      if ((cap == Capability.BINARY_ATTRIBUTES && hasNominalAtt) ||
+          (cap == Capability.UNARY_ATTRIBUTES && hasBinaryAtt) ||
+          (cap == Capability.EMPTY_NOMINAL_ATTRIBUTES && hasUnaryAtt) ||
+          (cap == Capability.BINARY_CLASS && hasNominalClass)) {
+        continue;
+      }
+      result.append(
+          indentStr + objectname + ".enable(" + capName + "." + cap.name() + ");\n");
       // dependency
       if (hasDependency(cap))
         result.append(
@@ -1611,6 +1675,6 @@ public class Capabilities
    * @return		the revision
    */
   public String getRevision() {
-    return RevisionUtils.extract("$Revision: 1.27 $");
+    return RevisionUtils.extract("$Revision: 7082 $");
   }
 }
