@@ -50,16 +50,16 @@ import java.text.ParseException;
  *
  * @author Mark Hall (mhall@cs.waikato.ac.nz)
  * @author FracPete (fracpete at waikato dot ac dot nz)
- * @version $Revision: 1.24 $
+ * @version $Revision: 7703 $
  * @see Loader
  */
-public class ArffLoader 
-  extends AbstractFileLoader 
+public class ArffLoader
+  extends AbstractFileLoader
   implements BatchConverter, IncrementalConverter, URLSourcedLoader {
 
   /** for serialization */
   static final long serialVersionUID = 2726929550544048587L;
-  
+
   /** the file extension */
   public static String FILE_EXTENSION = Instances.FILE_EXTENSION;
   public static String FILE_EXTENSION_COMPRESSED = FILE_EXTENSION + ".gz";
@@ -72,10 +72,10 @@ public class ArffLoader
 
   /** The parser for the ARFF file */
   protected transient ArffReader m_ArffReader = null;
-  
+
   /**
    * Reads data from an ARFF file, either in incremental or batch mode. <p/>
-   * 
+   *
    * Typical code for batch usage:
    * <pre>
    * BufferedReader reader = new BufferedReader(new FileReader("/some/where/file.arff"));
@@ -83,7 +83,7 @@ public class ArffLoader
    * Instances data = arff.getData();
    * data.setClassIndex(data.numAttributes() - 1);
    * </pre>
-   * 
+   *
    * Typical code for incremental usage:
    * <pre>
    * BufferedReader reader = new BufferedReader(new FileReader("/some/where/file.arff"));
@@ -95,18 +95,18 @@ public class ArffLoader
    *   data.add(inst);
    * }
    * </pre>
-   * 
+   *
    * @author  Eibe Frank (eibe@cs.waikato.ac.nz)
    * @author  Len Trigg (trigg@cs.waikato.ac.nz)
    * @author  fracpete (fracpete at waikato dot ac dot nz)
-   * @version $Revision: 1.24 $
+   * @version $Revision: 7703 $
    */
   public static class ArffReader
     implements RevisionHandler {
 
     /** the tokenizer for reading the stream */
     protected StreamTokenizer m_Tokenizer;
-    
+
     /** Buffer of values for sparse instance */
     protected double[] m_ValueBuffer;
 
@@ -118,11 +118,11 @@ public class ArffLoader
 
     /** the number of lines read so far */
     protected int m_Lines;
-    
+
     /**
      * Reads the data completely from the reader. The data can be accessed
      * via the <code>getData()</code> method.
-     * 
+     *
      * @param reader		the reader to use
      * @throws IOException	if something goes wrong
      * @see			#getData()
@@ -133,21 +133,21 @@ public class ArffLoader
 
       readHeader(1000);
       initBuffers();
-      
+
       Instance inst;
       while ((inst = readInstance(m_Data)) != null) {
         m_Data.add(inst);
       };
-      
+
       compactify();
     }
-    
+
     /**
      * Reads only the header and reserves the specified space for instances.
      * Further instances can be read via <code>readInstance()</code>.
-     * 
+     *
      * @param reader			the reader to use
-     * @param capacity 			the capacity of the new dataset 
+     * @param capacity 			the capacity of the new dataset
      * @throws IOException		if something goes wrong
      * @throws IllegalArgumentException	if capacity is negative
      * @see				#getStructure()
@@ -163,11 +163,11 @@ public class ArffLoader
       readHeader(capacity);
       initBuffers();
     }
-    
+
     /**
      * Reads the data without header according to the specified template.
      * The data can be accessed via the <code>getData()</code> method.
-     * 
+     *
      * @param reader		the reader to use
      * @param template		the template header
      * @param lines		the lines read so far
@@ -184,16 +184,16 @@ public class ArffLoader
 
       compactify();
     }
-    
+
     /**
-     * Initializes the reader without reading the header according to the 
-     * specified template. The data must be read via the 
+     * Initializes the reader without reading the header according to the
+     * specified template. The data must be read via the
      * <code>readInstance()</code> method.
-     * 
+     *
      * @param reader		the reader to use
      * @param template		the template header
      * @param lines		the lines read so far
-     * @param capacity 		the capacity of the new dataset 
+     * @param capacity 		the capacity of the new dataset
      * @throws IOException	if something goes wrong
      * @see			#getData()
      */
@@ -208,7 +208,7 @@ public class ArffLoader
 
     /**
      * initializes the buffers for sparse instances to be read
-     * 
+     *
      * @see			#m_ValueBuffer
      * @see			#m_IndicesBuffer
      */
@@ -216,7 +216,7 @@ public class ArffLoader
       m_ValueBuffer = new double[m_Data.numAttributes()];
       m_IndicesBuffer = new int[m_Data.numAttributes()];
     }
-    
+
     /**
      * compactifies the data
      */
@@ -224,7 +224,7 @@ public class ArffLoader
       if (m_Data != null)
         m_Data.compactify();
     }
-    
+
     /**
      * Throws error message with line number and last token read.
      *
@@ -242,13 +242,13 @@ public class ArffLoader
 
     /**
      * returns the current line number
-     * 
+     *
      * @return			the current line number
      */
     public int getLineNo() {
       return m_Lines + m_Tokenizer.lineno();
     }
-    
+
     /**
      * Gets next token, skipping empty lines.
      *
@@ -256,7 +256,7 @@ public class ArffLoader
      */
     protected void getFirstToken() throws IOException {
       while (m_Tokenizer.nextToken() == StreamTokenizer.TT_EOL) {};
-      
+
       if ((m_Tokenizer.ttype == '\'') ||
   	(m_Tokenizer.ttype == '"')) {
         m_Tokenizer.ttype = StreamTokenizer.TT_WORD;
@@ -279,7 +279,7 @@ public class ArffLoader
         errorMessage("premature end of file");
       }
     }
-    
+
     /**
      * Gets token and checks if its end of line.
      *
@@ -302,7 +302,7 @@ public class ArffLoader
     protected double getInstanceWeight() throws IOException {
       double weight = Double.NaN;
       m_Tokenizer.nextToken();
-      if (m_Tokenizer.ttype == StreamTokenizer.TT_EOL || 
+      if (m_Tokenizer.ttype == StreamTokenizer.TT_EOL ||
           m_Tokenizer.ttype == StreamTokenizer.TT_EOF) {
         return weight;
       }
@@ -346,13 +346,13 @@ public class ArffLoader
         m_Tokenizer.ttype = '?';
       }
     }
-  	
+
     /**
      * Initializes the StreamTokenizer used for reading the ARFF file.
      */
     protected void initTokenizer(){
-      m_Tokenizer.resetSyntax();         
-      m_Tokenizer.whitespaceChars(0, ' ');    
+      m_Tokenizer.resetSyntax();
+      m_Tokenizer.whitespaceChars(0, ' ');
       m_Tokenizer.wordChars(' '+1,'\u00FF');
       m_Tokenizer.whitespaceChars(',',',');
       m_Tokenizer.commentChar('%');
@@ -362,49 +362,49 @@ public class ArffLoader
       m_Tokenizer.ordinaryChar('}');
       m_Tokenizer.eolIsSignificant(true);
     }
-    
+
     /**
-     * Reads a single instance using the tokenizer and returns it. 
+     * Reads a single instance using the tokenizer and returns it.
      *
-     * @param structure 	the dataset header information, will get updated 
+     * @param structure 	the dataset header information, will get updated
      * 				in case of string or relational attributes
      * @return 			null if end of file has been reached
-     * @throws IOException 	if the information is not read 
+     * @throws IOException 	if the information is not read
      * successfully
-     */ 
+     */
     public Instance readInstance(Instances structure) throws IOException {
       return readInstance(structure, true);
     }
-    
+
     /**
-     * Reads a single instance using the tokenizer and returns it. 
+     * Reads a single instance using the tokenizer and returns it.
      *
-     * @param structure 	the dataset header information, will get updated 
+     * @param structure 	the dataset header information, will get updated
      * 				in case of string or relational attributes
-     * @param flag 		if method should test for carriage return after 
+     * @param flag 		if method should test for carriage return after
      * 				each instance
      * @return 			null if end of file has been reached
-     * @throws IOException 	if the information is not read 
+     * @throws IOException 	if the information is not read
      * successfully
-     */ 
+     */
     public Instance readInstance(Instances structure, boolean flag) throws IOException {
       return getInstance(structure, flag);
     }
-    
+
     /**
-     * Reads a single instance using the tokenizer and returns it. 
+     * Reads a single instance using the tokenizer and returns it.
      *
-     * @param structure 	the dataset header information, will get updated 
+     * @param structure 	the dataset header information, will get updated
      * 				in case of string or relational attributes
-     * @param flag 		if method should test for carriage return after 
+     * @param flag 		if method should test for carriage return after
      * 				each instance
      * @return 			null if end of file has been reached
-     * @throws IOException 	if the information is not read 
+     * @throws IOException 	if the information is not read
      * 				successfully
-     */ 
+     */
     protected Instance getInstance(Instances structure, boolean flag) throws IOException {
       m_Data = structure;
-      
+
       // Check if any attributes have been declared.
       if (m_Data.numAttributes() == 0) {
         errorMessage("no header information available");
@@ -415,7 +415,7 @@ public class ArffLoader
       if (m_Tokenizer.ttype == StreamTokenizer.TT_EOF) {
         return null;
       }
-      
+
       // Parse instance
       if (m_Tokenizer.ttype == '{') {
         return getInstanceSparse(flag);
@@ -427,15 +427,15 @@ public class ArffLoader
     /**
      * Reads a single instance using the tokenizer and returns it.
      *
-     * @param flag 		if method should test for carriage return after 
+     * @param flag 		if method should test for carriage return after
      * 				each instance
      * @return 			null if end of file has been reached
-     * @throws IOException 	if the information is not read 
+     * @throws IOException 	if the information is not read
      * 				successfully
-     */ 
+     */
     protected Instance getInstanceSparse(boolean flag) throws IOException {
       int valIndex, numValues = 0, maxIndex = -1;
-      
+
       // Get values
       do {
         // Get index
@@ -443,7 +443,7 @@ public class ArffLoader
         if (m_Tokenizer.ttype == '}') {
   	break;
         }
-   
+
         // Is index valid?
         try{
   	m_IndicesBuffer[numValues] = Integer.valueOf(m_Tokenizer.sval).intValue();
@@ -453,7 +453,7 @@ public class ArffLoader
         if (m_IndicesBuffer[numValues] <= maxIndex) {
   	errorMessage("indices have to be ordered");
         }
-        if ((m_IndicesBuffer[numValues] < 0) || 
+        if ((m_IndicesBuffer[numValues] < 0) ||
   	  (m_IndicesBuffer[numValues] >= m_Data.numAttributes())) {
   	errorMessage("index out of bounds");
         }
@@ -474,7 +474,7 @@ public class ArffLoader
           switch (m_Data.attribute(m_IndicesBuffer[numValues]).type()) {
             case Attribute.NOMINAL:
               // Check if value appears in header.
-              valIndex = 
+              valIndex =
                 m_Data.attribute(m_IndicesBuffer[numValues]).indexOfValue(m_Tokenizer.sval);
               if (valIndex == -1) {
                 errorMessage("nominal value not declared in header");
@@ -491,12 +491,12 @@ public class ArffLoader
   	  }
             break;
   	case Attribute.STRING:
-  	  m_ValueBuffer[numValues] = 
+  	  m_ValueBuffer[numValues] =
   	    m_Data.attribute(m_IndicesBuffer[numValues]).addStringValue(m_Tokenizer.sval);
             break;
           case Attribute.DATE:
             try {
-              m_ValueBuffer[numValues] = 
+              m_ValueBuffer[numValues] =
                 m_Data.attribute(m_IndicesBuffer[numValues]).parseDate(m_Tokenizer.sval);
             } catch (ParseException e) {
               errorMessage("unparseable date: " + m_Tokenizer.sval);
@@ -527,9 +527,9 @@ public class ArffLoader
           getLastToken(true);
         } else {
           weight = 1.0;
-        }        
+        }
       }
-        
+
       // Add instance to dataset
       double[] tempValues = new double[numValues];
       int[] tempIndices = new int[numValues];
@@ -537,30 +537,30 @@ public class ArffLoader
       System.arraycopy(m_IndicesBuffer, 0, tempIndices, 0, numValues);
       Instance inst = new SparseInstance(weight, tempValues, tempIndices, m_Data.numAttributes());
       inst.setDataset(m_Data);
-      
+
       return inst;
     }
 
     /**
      * Reads a single instance using the tokenizer and returns it.
      *
-     * @param flag 		if method should test for carriage return after 
+     * @param flag 		if method should test for carriage return after
      * 				each instance
      * @return 			null if end of file has been reached
-     * @throws IOException 	if the information is not read 
+     * @throws IOException 	if the information is not read
      * 				successfully
-     */ 
+     */
     protected Instance getInstanceFull(boolean flag) throws IOException {
       double[] instance = new double[m_Data.numAttributes()];
       int index;
-      
+
       // Get values for all attributes.
       for (int i = 0; i < m_Data.numAttributes(); i++){
         // Get next token
         if (i > 0) {
   	getNextToken();
         }
-              
+
         // Check if value is missing.
         if  (m_Tokenizer.ttype == '?') {
   	instance[i] = Instance.missingValue();
@@ -613,7 +613,7 @@ public class ArffLoader
   	}
         }
       }
-      
+
       double weight = 1.0;
       if (flag) {
         // check for an instance weight
@@ -624,26 +624,26 @@ public class ArffLoader
           weight = 1.0;
         }
       }
-        
+
       // Add instance to dataset
       Instance inst = new Instance(weight, instance);
       inst.setDataset(m_Data);
-      
+
       return inst;
     }
 
     /**
      * Reads and stores header of an ARFF file.
      *
-     * @param capacity 		the number of instances to reserve in the data 
+     * @param capacity 		the number of instances to reserve in the data
      * 				structure
-     * @throws IOException 	if the information is not read 
+     * @throws IOException 	if the information is not read
      * 				successfully
-     */ 
+     */
     protected void readHeader(int capacity) throws IOException {
       m_Lines = 0;
       String relationName = "";
-      
+
       // Get name of relation.
       getFirstToken();
       if (m_Tokenizer.ttype == StreamTokenizer.TT_EOF) {
@@ -659,7 +659,7 @@ public class ArffLoader
 
       // Create vectors to hold information temporarily.
       FastVector attributes = new FastVector();
-   
+
       // Get attribute declarations.
       getFirstToken();
       if (m_Tokenizer.ttype == StreamTokenizer.TT_EOF) {
@@ -674,12 +674,12 @@ public class ArffLoader
       if (!Instances.ARFF_DATA.equalsIgnoreCase(m_Tokenizer.sval)) {
         errorMessage("keyword " + Instances.ARFF_DATA + " expected");
       }
-      
+
       // Check if any attributes have been declared.
       if (attributes.size() == 0) {
         errorMessage("no attributes declared");
       }
-      
+
       m_Data = new Instances(relationName, attributes, capacity);
     }
 
@@ -688,7 +688,7 @@ public class ArffLoader
      *
      * @param attributes 		the current attributes vector
      * @return 			the new attributes vector
-     * @throws IOException 	if the information is not read 
+     * @throws IOException 	if the information is not read
      * 				successfully
      */
     protected FastVector parseAttribute(FastVector attributes) throws IOException {
@@ -699,10 +699,10 @@ public class ArffLoader
       getNextToken();
       attributeName = m_Tokenizer.sval;
       getNextToken();
-      
+
       // Check if attribute is nominal.
       if (m_Tokenizer.ttype == StreamTokenizer.TT_WORD) {
-        
+
         // Attribute is real, integer, or string.
         if (m_Tokenizer.sval.equalsIgnoreCase(Attribute.ARFF_ATTRIBUTE_REAL) ||
             m_Tokenizer.sval.equalsIgnoreCase(Attribute.ARFF_ATTRIBUTE_INTEGER) ||
@@ -729,15 +729,15 @@ public class ArffLoader
           }
           attributes.addElement(new Attribute(attributeName, format,
               attributes.size()));
-          
+
         } else if (m_Tokenizer.sval.equalsIgnoreCase(Attribute.ARFF_ATTRIBUTE_RELATIONAL)) {
           readTillEOL();
-          
+
           // Read attributes for subrelation
           // First, save current set of attributes
           FastVector atts = attributes;
           attributes = new FastVector();
-          
+
           // Now, read attributes until we hit end of declaration of relational value
           getFirstToken();
           if (m_Tokenizer.ttype == StreamTokenizer.TT_EOF) {
@@ -749,16 +749,16 @@ public class ArffLoader
             } else if (Attribute.ARFF_END_SUBRELATION.equalsIgnoreCase(m_Tokenizer.sval)) {
               getNextToken();
               if (!attributeName.equalsIgnoreCase(m_Tokenizer.sval)) {
-                errorMessage("declaration of subrelation " + attributeName + 
+                errorMessage("declaration of subrelation " + attributeName +
                       " must be terminated by " + "@end " + attributeName);
               }
               break;
             } else {
-              errorMessage("declaration of subrelation " + attributeName + 
+              errorMessage("declaration of subrelation " + attributeName +
                     " must be terminated by " + "@end " + attributeName);
             }
           } while (true);
-          
+
           // Make relation and restore original set of attributes
           Instances relation = new Instances(attributeName, attributes, 0);
           attributes = atts;
@@ -769,11 +769,11 @@ public class ArffLoader
                 "enumeration");
         }
       } else {
-        
+
         // Attribute is nominal.
         attributeValues = new FastVector();
         m_Tokenizer.pushBack();
-        
+
         // Get values for nominal attribute.
         if (m_Tokenizer.nextToken() != '{') {
           errorMessage("{ expected at beginning of enumeration");
@@ -793,7 +793,7 @@ public class ArffLoader
       getFirstToken();
       if (m_Tokenizer.ttype == StreamTokenizer.TT_EOF)
         errorMessage("premature end of file");
-      
+
       return attributes;
     }
 
@@ -804,35 +804,35 @@ public class ArffLoader
      */
     protected void readTillEOL() throws IOException {
       while (m_Tokenizer.nextToken() != StreamTokenizer.TT_EOL) {};
-      
+
       m_Tokenizer.pushBack();
     }
 
     /**
      * Returns the header format
-     * 
+     *
      * @return			the header format
      */
     public Instances getStructure() {
       return new Instances(m_Data, 0);
     }
-    
+
     /**
      * Returns the data that was read
-     * 
+     *
      * @return			the data
      */
     public Instances getData() {
       return m_Data;
     }
-    
+
     /**
      * Returns the revision string.
-     * 
+     *
      * @return		the revision
      */
     public String getRevision() {
-      return RevisionUtils.extract("$Revision: 1.24 $");
+      return RevisionUtils.extract("$Revision: 7703 $");
     }
   }
 
@@ -876,14 +876,15 @@ public class ArffLoader
   /**
    * Resets the Loader ready to read a new data set or the
    * same data set again.
-   * 
+   *
    * @throws IOException if something goes wrong
    */
   public void reset() throws IOException {
     m_structure = null;
+    m_ArffReader = null;
     setRetrieval(NONE);
-    
-    if (m_File != null && (new File(m_File)).isFile()) {
+
+    if (m_File != null && !(new File(m_File).isDirectory())) {
       setFile(new File(m_File));
     } else if (m_URL != null && !m_URL.equals("http://")) {
       setURL(m_URL);
@@ -891,7 +892,7 @@ public class ArffLoader
   }
 
   /**
-   * Resets the Loader object and sets the source of the data set to be 
+   * Resets the Loader object and sets the source of the data set to be
    * the supplied url.
    *
    * @param url the source url.
@@ -900,12 +901,15 @@ public class ArffLoader
   public void setSource(URL url) throws IOException {
     m_structure = null;
     setRetrieval(NONE);
-    
+
     setSource(url.openStream());
 
     m_URL = url.toString();
+    // make sure that the file is null so that any calls to
+    // reset() work properly
+    m_File = null;
   }
-  
+
 
   /**
    * get the File specified as the source
@@ -923,7 +927,7 @@ public class ArffLoader
    * @throws IOException if an error occurs
    */
   public void setFile(File file) throws IOException {
-    m_File = file.getAbsolutePath();
+    m_File = file.getPath();
     setSource(file);
   }
 
@@ -948,7 +952,7 @@ public class ArffLoader
   }
 
   /**
-   * Resets the Loader object and sets the source of the data set to be 
+   * Resets the Loader object and sets the source of the data set to be
    * the supplied InputStream.
    *
    * @param in the source InputStream.
@@ -962,7 +966,7 @@ public class ArffLoader
   }
 
   /**
-   * Determines and returns (if possible) the structure (internally the 
+   * Determines and returns (if possible) the structure (internally the
    * header) of the data set as an empty set of instances.
    *
    * @return the structure of the data set as an empty set of Instances
@@ -970,11 +974,10 @@ public class ArffLoader
    */
   public Instances getStructure() throws IOException {
 
-    if (m_sourceReader == null) {
-      throw new IOException("No source has been specified");
-    }
-
     if (m_structure == null) {
+      if (m_sourceReader == null) {
+        throw new IOException("No source has been specified");
+      }
       try {
 	m_ArffReader = new ArffReader(m_sourceReader, 1);
 	m_structure  = m_ArffReader.getStructure();
@@ -1009,25 +1012,26 @@ public class ArffLoader
 
     // Read all instances
     Instance inst;
+    Instances insts = new Instances(m_structure, 0);
     while ((inst = m_ArffReader.readInstance(m_structure)) != null)
-      m_structure.add(inst);
-    
-    Instances readIn = new Instances(m_structure);
+      insts.add(inst);
+
+    // Instances readIn = new Instances(m_structure);
 
     // close the stream
     m_sourceReader.close();
-    
-    return readIn;
+
+    return insts;
   }
 
   /**
-   * Read the data set incrementally---get the next instance in the data 
+   * Read the data set incrementally---get the next instance in the data
    * set or returns null if there are no
-   * more instances to get. If the structure hasn't yet been 
+   * more instances to get. If the structure hasn't yet been
    * determined by a call to getStructure then method should do so before
    * returning the next instance in the data set.
    *
-   * @param structure the dataset header information, will get updated in 
+   * @param structure the dataset header information, will get updated in
    * case of string or relational attributes
    * @return the next instance in the data set as an Instance object or null
    * if there are no more instances to be read
@@ -1042,11 +1046,15 @@ public class ArffLoader
     }
     setRetrieval(INCREMENTAL);
 
-    Instance current = m_ArffReader.readInstance(m_structure);
-    if (current == null) {
+    Instance current = null;
+    if (m_sourceReader != null)
+      current = m_ArffReader.readInstance(m_structure);
+
+    if ((m_sourceReader != null) && (current == null)) {
       try {
         // close the stream
         m_sourceReader.close();
+        m_sourceReader = null;
         //        reset();
       } catch (Exception ex) {
         ex.printStackTrace();
@@ -1054,14 +1062,14 @@ public class ArffLoader
     }
     return current;
   }
-  
+
   /**
    * Returns the revision string.
-   * 
+   *
    * @return		the revision
    */
   public String getRevision() {
-    return RevisionUtils.extract("$Revision: 1.24 $");
+    return RevisionUtils.extract("$Revision: 7703 $");
   }
 
   /**
