@@ -108,7 +108,7 @@ import java.util.Vector;
  *
  * @author Len Trigg (trigg@cs.waikato.ac.nz)
  * @author Eibe Frank (eibe@cs.waikato.ac.nz)
- * @version $Revision: 1.8 $
+ * @version $Revision: 6565 $
  */
 public class Discretize 
   extends Filter 
@@ -259,6 +259,7 @@ public class Discretize
    */
   public Capabilities getCapabilities() {
     Capabilities result = super.getCapabilities();
+    result.disableAll();
 
     // attributes
     result.enableAllAttributes();
@@ -926,7 +927,7 @@ public class Discretize
 	      attribValues.addElement("'("
 		      + Utils.doubleToString(m_CutPoints[i][j], 6) + "-inf)'");
 	      attributes.addElement(new Attribute(getInputFormat().
-						  attribute(i).name(),
+						  attribute(i).name() + "_" + (j+1),
 						  attribValues));
 	    }
 	  }
@@ -1014,7 +1015,7 @@ public class Discretize
    * @return		the revision
    */
   public String getRevision() {
-    return RevisionUtils.extract("$Revision: 1.8 $");
+    return RevisionUtils.extract("$Revision: 6565 $");
   }
 
   /**
