@@ -42,10 +42,12 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileReader;
 import java.io.Reader;
+import java.lang.reflect.Method;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Hashtable;
+import java.util.Locale;
 import java.util.Vector;
 
 import javax.swing.BorderFactory;
@@ -96,10 +98,31 @@ import weka.gui.visualize.VisualizePanel;
  * @author Len Trigg (trigg@cs.waikato.ac.nz)
  * @author Mark Hall (mhall@cs.waikato.ac.nz)
  * @author FracPete (fracpete at waikato dot ac dot nz)
- * @version $Revision: 4863 $
+ * @version $Revision: 7212 $
  */
 public class GUIChooser
   extends JFrame {
+  
+  static {
+    try {
+      Object MacApp = Class.forName("com.apple.eawt.Application").newInstance();
+      
+      Object macArffHandler = Class.forName("weka.gui.MacArffOpenFilesHandler").newInstance();
+      
+      Class fileHandlerClass = Class.forName("com.apple.eawt.OpenFilesHandler");
+      Class[] paramClass = new Class[1];
+      paramClass[0] = fileHandlerClass;
+      Object[] args = new Object[1];
+      args[0] = macArffHandler;
+      
+      Method m = MacApp.getClass().getMethod("setOpenFileHandler", paramClass);
+      System.out.println("Trying to install a file handler for Mac...");
+      m.invoke(MacApp, macArffHandler);
+      
+    } catch (Exception ex) {
+      // quietly ignore...
+    }
+  }
 
   /** for serialization */
   private static final long serialVersionUID = 9001529425230247914L;
@@ -120,25 +143,28 @@ public class GUIChooser
   protected JPanel m_PanelApplications = new JPanel();
   
   /** Click to open the Explorer */
-  protected JButton m_ExplorerBut = new JButton("Explorer");
+  protected JButton m_ExplorerBut = new JButton(Messages.getInstance().getString("GUIChooser_Explorer_Text"));
 
   /** The frame containing the explorer interface */
   protected JFrame m_ExplorerFrame;
 
   /** Click to open the Explorer */
-  protected JButton m_ExperimenterBut = new JButton("Experimenter");
+  protected JButton m_ExperimenterBut = new JButton(Messages.getInstance().getString("GUIChooser_Experimenter_Text"));
 
   /** The frame containing the experiment interface */
   protected JFrame m_ExperimenterFrame;
 
   /** Click to open the KnowledgeFlow */
-  protected JButton m_KnowledgeFlowBut = new JButton("KnowledgeFlow");
+  protected JButton m_KnowledgeFlowBut = new JButton(Messages.getInstance().getString("GUIChooser_KnowledgeFlow_Text"));
+  
+  /** Pending file to load on startup of the KnowledgeFlow */
+  protected String m_pendingKnowledgeFlowLoad = null;
 
   /** The frame containing the knowledge flow interface */
   protected JFrame m_KnowledgeFlowFrame;
 
   /** Click to open the simplecli */
-  protected JButton m_SimpleBut = new JButton("Simple CLI");
+  protected JButton m_SimpleBut = new JButton(Messages.getInstance().getString("GUIChooser_Simple_CLI_Text"));
   
   /** The SimpleCLI */
   protected SimpleCLI m_SimpleCLI;
@@ -187,7 +213,7 @@ public class GUIChooser
 
   /** The weka image */
   Image m_weka = Toolkit.getDefaultToolkit().
-    getImage(ClassLoader.getSystemResource("weka/gui/images/weka_background.gif"));
+    getImage(GUIChooser.class.getClassLoader().getResource("weka/gui/images/weka_background.gif"));
 
   /** filechooser for the TreeVisualizer */
   protected JFileChooser m_FileChooserTreeVisualizer = new JFileChooser(new File(System.getProperty("user.dir")));
@@ -207,42 +233,60 @@ public class GUIChooser
   /** contains the child frames (title &lt;-&gt; object). */
   protected HashSet<Container> m_ChildFrames = new HashSet<Container>();
   
+  /**
+   * Create a singleton instance of the GUIChooser 
+   */
+  public static synchronized void createSingleton() {
+    if (m_chooser == null) {
+      m_chooser = new GUIChooser();
+    }
+  }
+
+  /**
+   * Get the singleton instance of the GUIChooser
+   *
+   * @return the singleton instance of the GUIChooser
+   */
+  public static GUIChooser getSingleton() {
+    return m_chooser;
+  }
+  
   /**
    * Creates the experiment environment gui with no initial experiment
    */
   public GUIChooser() {
 
-    super("Weka GUI Chooser");
+    super(Messages.getInstance().getString("GUIChooser_Title_Text"));
     
     m_Self = this;
 
     // filechoosers
     m_FileChooserGraphVisualizer.addChoosableFileFilter(
-	  new ExtensionFileFilter(".bif", "BIF Files (*.bif)"));
+	  new ExtensionFileFilter(".bif", Messages.getInstance().getString("GUIChooser_BIF_Files_Text")));
     m_FileChooserGraphVisualizer.addChoosableFileFilter(
-	  new ExtensionFileFilter(".xml", "XML Files (*.xml)"));
+	  new ExtensionFileFilter(".xml", Messages.getInstance().getString("GUIChooser_XML_Files_Text")));
 
     m_FileChooserPlot.addChoosableFileFilter(
 	  new ExtensionFileFilter(
 	      Instances.FILE_EXTENSION,
-	      "ARFF Files (*" + Instances.FILE_EXTENSION + ")"));
+	      Messages.getInstance().getString("GUIChooser_ARFF_Files_Text_Front") + Instances.FILE_EXTENSION + Messages.getInstance().getString("GUIChooser_ARFF_Files_Text_End")));
     m_FileChooserPlot.setMultiSelectionEnabled(true);
     
     m_FileChooserROC.addChoosableFileFilter(
 	  new ExtensionFileFilter(
 	      Instances.FILE_EXTENSION,
-	      "ARFF Files (*" + Instances.FILE_EXTENSION + ")"));
+	      Messages.getInstance().getString("GUIChooser_ARFF_Files_Text_Front") + Instances.FILE_EXTENSION + Messages.getInstance().getString("GUIChooser_ARFF_Files_Text_End")));
 
     // general layout
     m_Icon = Toolkit.getDefaultToolkit().getImage(
-	ClassLoader.getSystemResource("weka/gui/weka_icon.gif"));
+	GUIChooser.class.getClassLoader().getResource("weka/gui/weka_icon_new_48.png"));
     setIconImage(m_Icon);
     this.getContentPane().setLayout(new BorderLayout());
     
     this.getContentPane().add(m_PanelApplications, BorderLayout.EAST);
-    
+
     // applications
-    m_PanelApplications.setBorder(BorderFactory.createTitledBorder("Applications"));
+    m_PanelApplications.setBorder(BorderFactory.createTitledBorder(Messages.getInstance().getString("GUIChooser_Applications_Text")));
     m_PanelApplications.setLayout(new GridLayout(4, 1));
     m_PanelApplications.add(m_ExplorerBut);
     m_PanelApplications.add(m_ExperimenterBut);
@@ -253,19 +297,14 @@ public class GUIChooser
     JPanel wekaPan = new JPanel();
     wekaPan.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
     wekaPan.setLayout(new BorderLayout());
-    wekaPan.setToolTipText("Weka, a native bird of New Zealand");
+    wekaPan.setToolTipText(Messages.getInstance().getString("GUIChooser_Weka_Native_Bird_Text"));
     ImageIcon wii = new ImageIcon(m_weka);
     JLabel wekaLab = new JLabel(wii);
     wekaPan.add(wekaLab, BorderLayout.CENTER);
-    String infoString = "<html>"
-      + "<font size=-2>"
-      + "Waikato Environment for Knowledge Analysis<br>"
-      + "Version " + Version.VERSION + "<br>"
-      + "(c) " + Copyright.getFromYear() + " - " + Copyright.getToYear() + "<br>"
-      + Copyright.getOwner() + "<br>"
-      + Copyright.getAddress()
-      + "</font>"
-      + "</html>";
+    String infoString = Messages.getInstance().getString("GUIChooser_Information_Text_Front") + " " + Version.VERSION + "<br>"
+      + "(c) " + Copyright.getFromYear() + " - " + Copyright.getToYear() + "<br>" + Copyright.getOwner() + "<br>"
+      + Copyright.getAddress() + Messages.getInstance().getString("GUIChooser_Information_Text_End");
+
     JLabel infoLab = new JLabel(infoString);
     infoLab.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
     wekaPan.add(infoLab, BorderLayout.SOUTH);
@@ -278,13 +317,13 @@ public class GUIChooser
     // Program
     m_jMenuProgram = new JMenu();
     m_jMenuBar.add(m_jMenuProgram);
-    m_jMenuProgram.setText("Program");
+    m_jMenuProgram.setText(Messages.getInstance().getString("GUIChooser_Program_Text"));
     m_jMenuProgram.setMnemonic('P');
     
     // Program/LogWindow
     JMenuItem jMenuItemProgramLogWindow = new JMenuItem();
     m_jMenuProgram.add(jMenuItemProgramLogWindow);
-    jMenuItemProgramLogWindow.setText("LogWindow");
+    jMenuItemProgramLogWindow.setText(Messages.getInstance().getString("GUIChooser_LogWindow_Text"));
     //jMenuItemProgramLogWindow.setMnemonic('L');
     jMenuItemProgramLogWindow.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, KeyEvent.CTRL_MASK));
     m_LogWindow.setIconImage(m_Icon);
@@ -296,7 +335,7 @@ public class GUIChooser
     
     final JMenuItem jMenuItemProgramMemUsage = new JMenuItem();
     m_jMenuProgram.add(jMenuItemProgramMemUsage);
-    jMenuItemProgramMemUsage.setText("Memory usage");
+    jMenuItemProgramMemUsage.setText(Messages.getInstance().getString("GUIChooser_Memory_Usage_Text"));
     //jMenuItemProgramMemUsage.setMnemonic('M');
     jMenuItemProgramMemUsage.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, KeyEvent.CTRL_MASK));
     
@@ -305,7 +344,7 @@ public class GUIChooser
         if (m_MemoryUsageFrame == null) {
           final MemoryUsagePanel panel = new MemoryUsagePanel(); 
           jMenuItemProgramMemUsage.setEnabled(false);
-          m_MemoryUsageFrame = new JFrame("Memory usage");
+          m_MemoryUsageFrame = new JFrame(Messages.getInstance().getString("GUIChooser_Memory_Usage_List_Text"));
           m_MemoryUsageFrame.setIconImage(m_Icon);
           m_MemoryUsageFrame.getContentPane().setLayout(new BorderLayout());
           m_MemoryUsageFrame.getContentPane().add(panel, BorderLayout.CENTER);
@@ -335,7 +374,7 @@ public class GUIChooser
     // Program/Exit
     JMenuItem jMenuItemProgramExit = new JMenuItem();
     m_jMenuProgram.add(jMenuItemProgramExit);
-    jMenuItemProgramExit.setText("Exit");
+    jMenuItemProgramExit.setText(Messages.getInstance().getString("GUIChooser_Exit_Text"));
 //    jMenuItemProgramExit.setMnemonic('E');
     jMenuItemProgramExit.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, KeyEvent.CTRL_MASK));
@@ -349,13 +388,13 @@ public class GUIChooser
     // Visualization
     m_jMenuVisualization = new JMenu();
     m_jMenuBar.add(m_jMenuVisualization);
-    m_jMenuVisualization.setText("Visualization");
+    m_jMenuVisualization.setText(Messages.getString("GUIChooser_Visualization_Text")); // "Visualization"
     m_jMenuVisualization.setMnemonic('V');
     
     // Visualization/Plot
     JMenuItem jMenuItemVisualizationPlot = new JMenuItem();
     m_jMenuVisualization.add(jMenuItemVisualizationPlot);
-    jMenuItemVisualizationPlot.setText("Plot");
+    jMenuItemVisualizationPlot.setText(Messages.getInstance().getString("GUIChooser_Plot_Text"));
     //jMenuItemVisualizationPlot.setMnemonic('P');
     jMenuItemVisualizationPlot.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, KeyEvent.CTRL_MASK));
@@ -376,7 +415,8 @@ public class GUIChooser
           if (j > 0)
             filenames += ", ";
           filenames += filename;
-          System.err.println("Loading instances from " + filename);
+          System.err.println(Messages.getInstance().getString("GUIChooser_LoadingInstancesFrom_Error_Text") + filename);
+          
           try {
             Reader r = new java.io.BufferedReader(new FileReader(filename));
             Instances i = new Instances(r);
@@ -384,10 +424,10 @@ public class GUIChooser
             PlotData2D pd1 = new PlotData2D(i);
 
             if (j == 0) {
-              pd1.setPlotName("Master plot");
+              pd1.setPlotName(Messages.getInstance().getString("GUIChooser_MasterPlot_Text"));
               panel.setMasterPlot(pd1);
             } else {
-              pd1.setPlotName("Plot "+(j+1));
+              pd1.setPlotName(Messages.getInstance().getString("GUIChooser_Plot_Text") + (j+1));
               pd1.m_useCustomColour = true;
               pd1.m_customColour = (j % 2 == 0) ? Color.red : Color.blue; 
               panel.addPlot(pd1);
@@ -396,13 +436,13 @@ public class GUIChooser
           catch (Exception ex) {
             ex.printStackTrace();
             JOptionPane.showMessageDialog(
-                m_Self, "Error loading file '" + files[j] + "':\n" + ex.getMessage());
+                m_Self, Messages.getInstance().getString("GUIChooser_Plot_LoadingFile_Error_Text_Front") + files[j] + Messages.getInstance().getString("GUIChooser_Plot_LoadingFile_Error_Text_End") + ex.getMessage());
             return;
           }
         }
 
         // create frame
-        final JFrame frame = new JFrame("Plot - " + filenames);
+        final JFrame frame = new JFrame(Messages.getInstance().getString("GUIChooser_Plot_PlotName_Text") + filenames);
         frame.setIconImage(m_Icon);
         frame.getContentPane().setLayout(new BorderLayout());
         frame.getContentPane().add(panel, BorderLayout.CENTER);
@@ -423,7 +463,7 @@ public class GUIChooser
     // Visualization/ROC
     JMenuItem jMenuItemVisualizationROC = new JMenuItem();
     m_jMenuVisualization.add(jMenuItemVisualizationROC);
-    jMenuItemVisualizationROC.setText("ROC");
+    jMenuItemVisualizationROC.setText(Messages.getInstance().getString("GUIChooser_ROC_Text"));
     // jMenuItemVisualizationROC.setMnemonic('R');
     jMenuItemVisualizationROC.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, KeyEvent.CTRL_MASK));
@@ -444,13 +484,13 @@ public class GUIChooser
         catch (Exception ex) {
           ex.printStackTrace();
           JOptionPane.showMessageDialog(
-              m_Self, "Error loading file '" + filename + "':\n" + ex.getMessage());
+              m_Self, Messages.getInstance().getString("GUIChooser_ROC_LoadingFile_Error_Text_Front") + filename + Messages.getInstance().getString("GUIChooser_ROC_LoadingFile_Error_Text_End") + ex.getMessage());
           return;
         }
         result.setClassIndex(result.numAttributes() - 1);
         ThresholdVisualizePanel vmc = new ThresholdVisualizePanel();
-        vmc.setROCString("(Area under ROC = " + 
-            Utils.doubleToString(ThresholdCurve.getROCArea(result), 4) + ")");
+        vmc.setROCString(Messages.getInstance().getString("GUIChooser_ROC_AreaUnderROC_Text_Front") + 
+            Utils.doubleToString(ThresholdCurve.getROCArea(result), 4) + Messages.getInstance().getString("GUIChooser_ROC_AreaUnderROC_Text_End"));
         vmc.setName(result.relationName());
         PlotData2D tempd = new PlotData2D(result);
         tempd.setPlotName(result.relationName());
@@ -461,11 +501,11 @@ public class GUIChooser
         catch (Exception ex) {
           ex.printStackTrace();
           JOptionPane.showMessageDialog(
-              m_Self, "Error adding plot:\n" + ex.getMessage());
+              m_Self, Messages.getInstance().getString("GUIChooser_ROC_AddingPlot_Error_Text") + ex.getMessage());
           return;
         }
 
-        final JFrame frame = new JFrame("ROC - " + filename);
+        final JFrame frame = new JFrame(Messages.getInstance().getString("GUIChooser_ROC_File_Text") + filename);
         frame.setIconImage(m_Icon);
         frame.getContentPane().setLayout(new BorderLayout());
         frame.getContentPane().add(vmc, BorderLayout.CENTER);
@@ -486,7 +526,7 @@ public class GUIChooser
     // Visualization/TreeVisualizer
     JMenuItem jMenuItemVisualizationTree = new JMenuItem();
     m_jMenuVisualization.add(jMenuItemVisualizationTree);
-    jMenuItemVisualizationTree.setText("TreeVisualizer");
+    jMenuItemVisualizationTree.setText(Messages.getInstance().getString("GUIChooser_TreeVisualizer_Text"));
     // jMenuItemVisualizationTree.setMnemonic('T');
     jMenuItemVisualizationTree.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, KeyEvent.CTRL_MASK));
@@ -509,12 +549,12 @@ public class GUIChooser
         catch (Exception ex) {
           ex.printStackTrace();
           JOptionPane.showMessageDialog(
-              m_Self, "Error loading file '" + filename + "':\n" + ex.getMessage());
+              m_Self, Messages.getInstance().getString("GUIChooser_TreeVisualizer_LoadingFile_Error_Text_Front") + filename + Messages.getInstance().getString("GUIChooser_TreeVisualizer_LoadingFile_Error_Text_End") + ex.getMessage());
           return;
         }
 
         // create frame
-        final JFrame frame = new JFrame("TreeVisualizer - " + filename);
+        final JFrame frame = new JFrame(Messages.getInstance().getString("GUIChooser_TreeVisualizer_File_Text") + filename);
         frame.setIconImage(m_Icon);
         frame.getContentPane().setLayout(new BorderLayout());
         frame.getContentPane().add(new TreeVisualizer(null, top, arrange), BorderLayout.CENTER);
@@ -535,7 +575,7 @@ public class GUIChooser
     // Visualization/GraphVisualizer
     JMenuItem jMenuItemVisualizationGraph = new JMenuItem();
     m_jMenuVisualization.add(jMenuItemVisualizationGraph);
-    jMenuItemVisualizationGraph.setText("GraphVisualizer");
+    jMenuItemVisualizationGraph.setText(Messages.getInstance().getString("GUIChooser_GraphVisualizer_Text"));
  //   jMenuItemVisualizationGraph.setMnemonic('G');
     jMenuItemVisualizationGraph.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, KeyEvent.CTRL_MASK));
@@ -562,12 +602,12 @@ public class GUIChooser
         catch (Exception ex) {
           ex.printStackTrace();
           JOptionPane.showMessageDialog(
-              m_Self, "Error loading file '" + filename + "':\n" + ex.getMessage());
+             m_Self, Messages.getInstance().getString("GUIChooser_GraphVisualizer_LoadingFile_Error_Text_Front") + filename + Messages.getInstance().getString("GUIChooser_GraphVisualizer_LoadingFile_Error_Text_End") + ex.getMessage());
           return;
         }
 
         // create frame
-        final JFrame frame = new JFrame("GraphVisualizer - " + filename);
+        final JFrame frame = new JFrame(Messages.getInstance().getString("GUIChooser_GraphVisualizer_File_Text") + filename);
         frame.setIconImage(m_Icon);
         frame.getContentPane().setLayout(new BorderLayout());
         frame.getContentPane().add(panel, BorderLayout.CENTER);
@@ -588,7 +628,7 @@ public class GUIChooser
     // Visualization/BoundaryVisualizer
     final JMenuItem jMenuItemVisualizationBoundary = new JMenuItem();
     m_jMenuVisualization.add(jMenuItemVisualizationBoundary);
-    jMenuItemVisualizationBoundary.setText("BoundaryVisualizer");
+    jMenuItemVisualizationBoundary.setText(Messages.getInstance().getString("GUIChooser_BoundaryVisualizer_Text"));
     // jMenuItemVisualizationBoundary.setMnemonic('B');
     jMenuItemVisualizationBoundary.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, KeyEvent.CTRL_MASK));
@@ -597,12 +637,16 @@ public class GUIChooser
       public void actionPerformed(ActionEvent e) {
         if (m_BoundaryVisualizerFrame == null) {
           jMenuItemVisualizationBoundary.setEnabled(false);
-          m_BoundaryVisualizerFrame = new JFrame("BoundaryVisualizer");
+          m_BoundaryVisualizerFrame = new JFrame(Messages.getInstance().getString("GUIChooser_BoundaryVisualizer_JFrame_Text"));
           m_BoundaryVisualizerFrame.setIconImage(m_Icon);
           m_BoundaryVisualizerFrame.getContentPane().setLayout(new BorderLayout());
-          m_BoundaryVisualizerFrame.getContentPane().add(new BoundaryVisualizer(), BorderLayout.CENTER);
+          
+          final BoundaryVisualizer bv = new BoundaryVisualizer();
+          m_BoundaryVisualizerFrame.getContentPane().add(bv, BorderLayout.CENTER);
+          m_BoundaryVisualizerFrame.setSize(bv.getMinimumSize());
           m_BoundaryVisualizerFrame.addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent w) {
+              bv.stopPlotting();
               m_BoundaryVisualizerFrame.dispose();
               m_BoundaryVisualizerFrame = null;
               jMenuItemVisualizationBoundary.setEnabled(true);
@@ -610,7 +654,8 @@ public class GUIChooser
             }
           });
           m_BoundaryVisualizerFrame.pack();
-          m_BoundaryVisualizerFrame.setSize(800, 600);
+          //m_BoundaryVisualizerFrame.setSize(800, 600);
+          m_BoundaryVisualizerFrame.setResizable(false);
           m_BoundaryVisualizerFrame.setVisible(true);
           // dont' do a System.exit after last window got closed!
           BoundaryVisualizer.setExitIfNoWindowsOpen(false);
@@ -619,7 +664,7 @@ public class GUIChooser
     });
     
     // Extensions
-    JMenu jMenuExtensions = new JMenu("Extensions");
+    JMenu jMenuExtensions = new JMenu(Messages.getInstance().getString("GUIChooser_Extensions_Text"));
     jMenuExtensions.setMnemonic(java.awt.event.KeyEvent.VK_E);
     m_jMenuBar.add(jMenuExtensions);
     jMenuExtensions.setVisible(false);
@@ -687,13 +732,13 @@ public class GUIChooser
     // Tools
     m_jMenuTools = new JMenu();
     m_jMenuBar.add(m_jMenuTools);
-    m_jMenuTools.setText("Tools");
+    m_jMenuTools.setText(Messages.getInstance().getString("GUIChooser_Tools_Text"));
     m_jMenuTools.setMnemonic('T');
     
     // Tools/ArffViewer
     JMenuItem jMenuItemToolsArffViewer = new JMenuItem();
     m_jMenuTools.add(jMenuItemToolsArffViewer);
-    jMenuItemToolsArffViewer.setText("ArffViewer");
+    jMenuItemToolsArffViewer.setText(Messages.getInstance().getString("GUIChooser_ArffViewer_Text"));
     // jMenuItemToolsArffViewer.setMnemonic('A');
     jMenuItemToolsArffViewer.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, KeyEvent.CTRL_MASK));
@@ -715,7 +760,7 @@ public class GUIChooser
     // Tools/SqlViewer
     final JMenuItem jMenuItemToolsSql = new JMenuItem();
     m_jMenuTools.add(jMenuItemToolsSql);
-    jMenuItemToolsSql.setText("SqlViewer");
+    jMenuItemToolsSql.setText(Messages.getInstance().getString("GUIChooser_SqlViewer_Text"));
     // jMenuItemToolsSql.setMnemonic('S');
     jMenuItemToolsSql.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_MASK));
@@ -725,7 +770,7 @@ public class GUIChooser
         if (m_SqlViewerFrame == null) {
           jMenuItemToolsSql.setEnabled(false);
           final SqlViewer sql = new SqlViewer(null);
-          m_SqlViewerFrame = new JFrame("SqlViewer");
+          m_SqlViewerFrame = new JFrame(Messages.getInstance().getString("GUIChooser_SqlViewer_JFrame_Text"));
           m_SqlViewerFrame.setIconImage(m_Icon);
           m_SqlViewerFrame.getContentPane().setLayout(new BorderLayout());
           m_SqlViewerFrame.getContentPane().add(sql, BorderLayout.CENTER);
@@ -747,7 +792,7 @@ public class GUIChooser
     // Tools/Bayes net editor
     final JMenuItem jMenuItemBayesNet = new JMenuItem();
     m_jMenuTools.add(jMenuItemBayesNet);
-    jMenuItemBayesNet.setText("Bayes net editor");
+    jMenuItemBayesNet.setText(Messages.getInstance().getString("GUIChooser_BayesNetEditor_Text"));
     jMenuItemBayesNet.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, KeyEvent.CTRL_MASK));
     jMenuItemBayesNet.addActionListener(new ActionListener() {
@@ -756,7 +801,7 @@ public class GUIChooser
           jMenuItemBayesNet.setEnabled(false);
           final GUI bayesNetGUI = new GUI();
           JMenuBar bayesBar = bayesNetGUI.getMenuBar();
-          m_BayesNetGUIFrame = new JFrame("Bayes Network Editor");
+          m_BayesNetGUIFrame = new JFrame(Messages.getInstance().getString("GUIChooser_BayesNetworkEditor_JFrame_Text"));
           m_BayesNetGUIFrame.setIconImage(m_Icon);
           m_BayesNetGUIFrame.setJMenuBar(bayesBar);
           m_BayesNetGUIFrame.getContentPane().add(bayesNetGUI, BorderLayout.CENTER);
@@ -777,19 +822,19 @@ public class GUIChooser
     // Help
     m_jMenuHelp = new JMenu();
     m_jMenuBar.add(m_jMenuHelp);
-    m_jMenuHelp.setText("Help");
+    m_jMenuHelp.setText(Messages.getInstance().getString("GUIChooser_Help_Text"));
     m_jMenuHelp.setMnemonic('H');
     
     // Help/Homepage
     JMenuItem jMenuItemHelpHomepage = new JMenuItem();
     m_jMenuHelp.add(jMenuItemHelpHomepage);
-    jMenuItemHelpHomepage.setText("Weka homepage");
+    jMenuItemHelpHomepage.setText(Messages.getInstance().getString("GUIChooser_WekaHomepage_Text"));
     // jMenuItemHelpHomepage.setMnemonic('H');
     jMenuItemHelpHomepage.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, KeyEvent.CTRL_MASK));
     jMenuItemHelpHomepage.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
-        BrowserHelper.openURL("http://www.cs.waikato.ac.nz/~ml/weka/");
+        BrowserHelper.openURL(Messages.getInstance().getString("GUIChooser_WekaHomepage_URL_Text"));
       }
     });
     
@@ -798,35 +843,35 @@ public class GUIChooser
     // Help/WekaWiki
     JMenuItem jMenuItemHelpWekaWiki = new JMenuItem();
     m_jMenuHelp.add(jMenuItemHelpWekaWiki);
-    jMenuItemHelpWekaWiki.setText("HOWTOs, code snippets, etc.");
+    jMenuItemHelpWekaWiki.setText(Messages.getInstance().getString("GUIChooser_WekaHOWTO_Text"));
     // jMenuItemHelpWekaWiki.setMnemonic('W');
     jMenuItemHelpWekaWiki.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, KeyEvent.CTRL_MASK));
     
     jMenuItemHelpWekaWiki.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
-        BrowserHelper.openURL("http://weka.wiki.sourceforge.net/");
+        BrowserHelper.openURL(Messages.getInstance().getString("GUIChooser_WekaWiki_URL_Text"));
       }
     });
     
     // Help/Sourceforge
     JMenuItem jMenuItemHelpSourceforge = new JMenuItem();
     m_jMenuHelp.add(jMenuItemHelpSourceforge);
-    jMenuItemHelpSourceforge.setText("Weka on Sourceforge");
+    jMenuItemHelpSourceforge.setText(Messages.getInstance().getString("GUIChooser_WekaOnSourceforge_Text"));
 //    jMenuItemHelpSourceforge.setMnemonic('F');
     jMenuItemHelpSourceforge.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.CTRL_MASK));
     
     jMenuItemHelpSourceforge.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
-        BrowserHelper.openURL("http://sourceforge.net/projects/weka/");
+        BrowserHelper.openURL(Messages.getInstance().getString("GUIChooser_WekaOnSourceforge_URL_Text"));
       }
     });
     
     // Help/SystemInfo
     final JMenuItem jMenuItemHelpSysInfo = new JMenuItem();
     m_jMenuHelp.add(jMenuItemHelpSysInfo);
-    jMenuItemHelpSysInfo.setText("SystemInfo");
+    jMenuItemHelpSysInfo.setText(Messages.getInstance().getString("GUIChooser_SystemInfo_Text"));
 //    jMenuItemHelpSysInfo.setMnemonic('S');
     jMenuItemHelpSysInfo.
       setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, KeyEvent.CTRL_MASK));
@@ -835,7 +880,7 @@ public class GUIChooser
       public void actionPerformed(ActionEvent e) {
         if (m_SystemInfoFrame == null) {
           jMenuItemHelpSysInfo.setEnabled(false);
-          m_SystemInfoFrame = new JFrame("SystemInfo");
+          m_SystemInfoFrame = new JFrame(Messages.getInstance().getString("GUIChooser_SystemInfo_JFrame_Text"));
           m_SystemInfoFrame.setIconImage(m_Icon);
           m_SystemInfoFrame.getContentPane().setLayout(new BorderLayout());
 
@@ -855,7 +900,8 @@ public class GUIChooser
             data[i][0] = names.get(i).toString();
             data[i][1] = info.get(data[i][0]).toString();
           }
-          String[] titles = new String[]{"Key", "Value"};
+   
+          String[] titles = new String[]{Messages.getInstance().getString("GUIChooser_SystemInfo_TitleKey_Text"), Messages.getInstance().getString("GUIChooser_SystemInfo_TitleValue_Text")};
           JTable table = new JTable(data, titles);
 
           m_SystemInfoFrame.getContentPane().add(new JScrollPane(table), BorderLayout.CENTER);
@@ -879,32 +925,17 @@ public class GUIChooser
 
     m_ExplorerBut.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
-	if (m_ExplorerFrame == null) {
-	  m_ExplorerBut.setEnabled(false);
-	  m_ExplorerFrame = new JFrame("Weka Explorer");
-	  m_ExplorerFrame.setIconImage(m_Icon);
-	  m_ExplorerFrame.getContentPane().setLayout(new BorderLayout());
-	  m_ExplorerFrame.getContentPane().add(new Explorer(), BorderLayout.CENTER);
-	  m_ExplorerFrame.addWindowListener(new WindowAdapter() {
-	    public void windowClosing(WindowEvent w) {
-	      m_ExplorerFrame.dispose();
-	      m_ExplorerFrame = null;
-	      m_ExplorerBut.setEnabled(true);
-	      checkExit();
-	    }
-	  });
-	  m_ExplorerFrame.pack();
-	  m_ExplorerFrame.setSize(800, 600);
-	  m_ExplorerFrame.setVisible(true);
-	}
+        showExplorer(null);
       }
     });
 
     m_ExperimenterBut.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
 	if (m_ExperimenterFrame == null) {
+		System.out.println("CALLED ExperimentEnvironment");
+		
 	  m_ExperimenterBut.setEnabled(false);
-	  m_ExperimenterFrame = new JFrame("Weka Experiment Environment");
+	  m_ExperimenterFrame = new JFrame(Messages.getInstance().getString("GUIChooser_WekaExperimentEnvironment_JFrame_Text"));
 	  m_ExperimenterFrame.setIconImage(m_Icon);
 	  m_ExperimenterFrame.getContentPane().setLayout(new BorderLayout());
 	  m_ExperimenterFrame.getContentPane()
@@ -929,7 +960,13 @@ public class GUIChooser
           if (m_KnowledgeFlowFrame == null) {
             final KnowledgeFlowApp kna = KnowledgeFlowApp.getSingleton();
             m_KnowledgeFlowBut.setEnabled(false);
-            m_KnowledgeFlowFrame = new JFrame("Weka KnowledgeFlow Environment");
+            if (m_pendingKnowledgeFlowLoad != null && 
+                m_pendingKnowledgeFlowLoad.length() > 0) {
+              KnowledgeFlowApp.getSingleton().
+                loadInitialLayout(m_pendingKnowledgeFlowLoad);
+              m_pendingKnowledgeFlowLoad = null;
+            }
+            m_KnowledgeFlowFrame = new JFrame(Messages.getInstance().getString("GUIChooser_WekaKnowledgeFlowEnvironment_JFrame_Text"));
             m_KnowledgeFlowFrame.setIconImage(m_Icon);
             m_KnowledgeFlowFrame.getContentPane().setLayout(new BorderLayout());
             m_KnowledgeFlowFrame.getContentPane()
@@ -952,7 +989,7 @@ public class GUIChooser
 
     m_KnowledgeFlowBut.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
-        KnowledgeFlow.startApp();
+        showKnowledgeFlow(null);
       }
     });
 
@@ -964,7 +1001,7 @@ public class GUIChooser
             m_SimpleCLI = new SimpleCLI();
             m_SimpleCLI.setIconImage(m_Icon);
           } catch (Exception ex) {
-            throw new Error("Couldn't start SimpleCLI!");
+            throw new Error(Messages.getInstance().getString("GUIChooser_UnableToStartSimpleCLI_Error_Text"));
           }
           m_SimpleCLI.addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent w) {
@@ -1016,6 +1053,57 @@ public class GUIChooser
     pack();
   }
   
+  public void showKnowledgeFlow(String fileToLoad) {
+    if (m_KnowledgeFlowFrame == null) {
+      KnowledgeFlow.startApp();
+      m_pendingKnowledgeFlowLoad = fileToLoad;
+    } else {
+      if (fileToLoad != null) {
+        KnowledgeFlowApp.getSingleton().loadInitialLayout(fileToLoad);
+      }
+    }
+  }
+  
+  public void showExplorer(String fileToLoad) {
+    Explorer expl = null;
+    if (m_ExplorerFrame == null) {
+      m_ExplorerBut.setEnabled(false);
+      m_ExplorerFrame = new JFrame("Weka Explorer");
+      m_ExplorerFrame.setIconImage(m_Icon);
+      m_ExplorerFrame.getContentPane().setLayout(new BorderLayout());
+      expl = new Explorer();
+
+      m_ExplorerFrame.getContentPane().add(expl, BorderLayout.CENTER);
+      m_ExplorerFrame.addWindowListener(new WindowAdapter() {
+          public void windowClosing(WindowEvent w) {
+            m_ExplorerFrame.dispose();
+            m_ExplorerFrame = null;
+            m_ExplorerBut.setEnabled(true);
+            checkExit();
+          }
+        });
+      m_ExplorerFrame.pack();
+      m_ExplorerFrame.setSize(800, 600);
+      m_ExplorerFrame.setVisible(true);
+    } else {
+      Object o = m_ExplorerFrame.getContentPane().getComponent(0);
+      if (o instanceof Explorer) {
+        expl = (Explorer)o;
+      }
+    }
+
+    if (fileToLoad != null) {
+      try {
+        weka.core.converters.AbstractFileLoader loader = 
+          weka.core.converters.ConverterUtils.getLoaderForFile(fileToLoad);
+        loader.setFile(new File(fileToLoad));
+        expl.getPreprocessPanel().setInstancesFromFile(loader);
+      } catch (Exception ex) {
+        ex.printStackTrace();
+      }
+    }
+  }
+  
   /**
    * insert the menu item in a sorted fashion.
    * 
@@ -1131,7 +1219,7 @@ public class GUIChooser
    * Specialized JFrame class.
    * 
    * @author  fracpete (fracpete at waikato dot ac dot nz)
-   * @version $Revision: 4863 $
+   * @version $Revision: 7212 $
    */
   public static class ChildFrameSDI 
     extends JFrame {
@@ -1198,9 +1286,9 @@ public class GUIChooser
   protected void createTitle(String title) {
     String      newTitle;
     
-    newTitle = "Weka " + new Version();
+    newTitle = Messages.getInstance().getString("GUIChooser_CreateTitle_Text_Front") + new Version();
     if (title.length() != 0)
-      newTitle += " - " + title;
+      newTitle += Messages.getInstance().getString("GUIChooser_CreateTitle_Text_End") + title;
     
     setTitle(newTitle);
   }
@@ -1267,7 +1355,7 @@ public class GUIChooser
    */
   public static void main(String [] args) {
 
-    weka.core.logging.Logger.log(weka.core.logging.Logger.Level.INFO, "Logging started");
+    weka.core.logging.Logger.log(weka.core.logging.Logger.Level.INFO, Messages.getInstance().getString("GUIChooser_Main_LoggingStarted_Text"));
     LookAndFeel.setLookAndFeel();
     
     try {
@@ -1275,8 +1363,13 @@ public class GUIChooser
       // uncomment to disable the memory management:
       //m_Memory.setEnabled(false);
 
-      m_chooser = new GUIChooser();
+      //m_chooser = new GUIChooser();
+      GUIChooser.createSingleton();
       m_chooser.setVisible(true);
+      
+      if (args != null && args.length > 0) {
+        m_chooser.showExplorer(args[0]);
+      }
 
       Thread memMonitor = new Thread() {
         public void run() {
@@ -1323,9 +1416,9 @@ public class GUIChooser
                 // display error
                 m_chooser.m_LogWindow.setVisible(true);
                 m_chooser.m_LogWindow.toFront();
-                System.err.println("\ndisplayed message:");
+                System.err.println(Messages.getInstance().getString("GUIChooser_Main_Error_Text_Front"));
                 m_Memory.showOutOfMemory();
-                System.err.println("\nexiting...");
+                System.err.println(Messages.getInstance().getString("GUIChooser_Main_Error_Text_End"));
                 System.exit(-1);
               }
             } 
