@@ -113,7 +113,7 @@ import javax.swing.event.InternalFrameEvent;
  <!-- options-end -->
  *
  * @author  fracpete (fracpete at waikato dot ac dot nz)
- * @version $Revision: 4718 $
+ * @version $Revision: 7193 $
  */
 public class Main
   extends JFrame
@@ -126,7 +126,7 @@ public class Main
    * DesktopPane with background image.
    * 
    * @author  fracpete (fracpete at waikato dot ac dot nz)
-   * @version $Revision: 4718 $
+   * @version $Revision: 7193 $
    */
   public static class BackgroundDesktopPane
     extends JDesktopPane {
@@ -178,7 +178,7 @@ public class Main
    * Specialized JFrame class.
    * 
    * @author  fracpete (fracpete at waikato dot ac dot nz)
-   * @version $Revision: 4718 $
+   * @version $Revision: 7193 $
    */
   public static class ChildFrameSDI 
     extends JFrame {
@@ -241,7 +241,7 @@ public class Main
    * Specialized JInternalFrame class.
    * 
    * @author  fracpete (fracpete at waikato dot ac dot nz)
-   * @version $Revision: 4718 $
+   * @version $Revision: 7193 $
    */
   public static class ChildFrameMDI
     extends JInternalFrame {
@@ -306,8 +306,8 @@ public class Main
   public final static int GUI_SDI = 1;
   /** GUI tags. */
   public static final Tag[] TAGS_GUI = {
-    new Tag(GUI_MDI, "MDI", "MDI Layout"),
-    new Tag(GUI_SDI, "SDI", "SDI Layout")
+    new Tag(GUI_MDI, "MDI", Messages.getInstance().getString("Main_Tag_GUI_Text_First")),
+    new Tag(GUI_SDI, "SDI", Messages.getInstance().getString("Main_Tag_GUI_Text_Second"))
   };
   
   /** the frame itself. */
@@ -549,24 +549,24 @@ public class Main
       // main window
       createTitle("");
       this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
-      this.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("weka/gui/weka_icon.gif")).getImage());
+      this.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("weka/gui/weka_icon_new_48.png")).getImage());
 
       // bits and pieces
       m_FileChooserGraphVisualizer.addChoosableFileFilter(
-	  new ExtensionFileFilter(".bif", "BIF Files (*.bif)"));
+	  new ExtensionFileFilter(".bif", Messages.getInstance().getString("Main_InitGUI_ExtensionFileFilter_Text_First")));
       m_FileChooserGraphVisualizer.addChoosableFileFilter(
-	  new ExtensionFileFilter(".xml", "XML Files (*.xml)"));
+	  new ExtensionFileFilter(".xml", Messages.getInstance().getString("Main_InitGUI_ExtensionFileFilter_Text_Second")));
 
       m_FileChooserPlot.addChoosableFileFilter(
 	  new ExtensionFileFilter(
 	      Instances.FILE_EXTENSION,
-	      "ARFF Files (*" + Instances.FILE_EXTENSION + ")"));
+	      Messages.getInstance().getString("Main_InitGUI_ExtensionFileFilter_Text_Third") + Instances.FILE_EXTENSION + ")"));
       m_FileChooserPlot.setMultiSelectionEnabled(true);
 
       m_FileChooserROC.addChoosableFileFilter(
 	  new ExtensionFileFilter(
 	      Instances.FILE_EXTENSION,
-	      "ARFF Files (*" + Instances.FILE_EXTENSION + ")"));
+	      Messages.getInstance().getString("Main_InitGUI_ExtensionFileFilter_Text_Third") + Instances.FILE_EXTENSION + ")"));
 
       // Desktop
       if (m_GUIType == GUI_MDI) {
@@ -585,7 +585,7 @@ public class Main
       // Program
       jMenuProgram = new JMenu();
       jMenuBar.add(jMenuProgram);
-      jMenuProgram.setText("Program");
+      jMenuProgram.setText(Messages.getInstance().getString("Main_InitGUI_JMenuProgram_SetText_Text"));
       jMenuProgram.setMnemonic('P');
 
       // Program/Preferences
@@ -606,7 +606,7 @@ public class Main
       // Program/LogWindow
       jMenuItemProgramLogWindow = new JMenuItem();
       jMenuProgram.add(jMenuItemProgramLogWindow);
-      jMenuItemProgramLogWindow.setText("LogWindow");
+      jMenuItemProgramLogWindow.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemProgramLogWindow_SetText_Text"));
       jMenuItemProgramLogWindow.setMnemonic('L');
       jMenuItemProgramLogWindow.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -616,7 +616,7 @@ public class Main
       
       jMenuItemProgramMemoryUsage = new JMenuItem();
       jMenuProgram.add(jMenuItemProgramMemoryUsage);
-      jMenuItemProgramMemoryUsage.setText("Memory usage");
+      jMenuItemProgramMemoryUsage.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemProgramMemoryUsage_SetText_Text"));
       jMenuItemProgramMemoryUsage.setMnemonic('M');
       jMenuItemProgramMemoryUsage.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -666,7 +666,7 @@ public class Main
       // Program/Exit
       jMenuItemProgramExit = new JMenuItem();
       jMenuProgram.add(jMenuItemProgramExit);
-      jMenuItemProgramExit.setText("Exit");
+      jMenuItemProgramExit.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemProgramExit_SetText_Text"));
       jMenuItemProgramExit.setMnemonic('E');
       jMenuItemProgramExit.addActionListener(new ActionListener() {	
 	public void actionPerformed(ActionEvent evt) {
@@ -694,13 +694,13 @@ public class Main
       // Applications
       jMenuApplications = new JMenu();
       jMenuBar.add(jMenuApplications);
-      jMenuApplications.setText("Applications");
+      jMenuApplications.setText(Messages.getInstance().getString("Main_InitGUI_JMenuApplications_SetText_Text"));
       jMenuApplications.setMnemonic('A');
 
       // Applications/Explorer
       jMenuItemApplicationsExplorer = new JMenuItem();
       jMenuApplications.add(jMenuItemApplicationsExplorer);
-      jMenuItemApplicationsExplorer.setText("Explorer");
+      jMenuItemApplicationsExplorer.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemApplicationsExplorer_SetText_Text"));
       jMenuItemApplicationsExplorer.setMnemonic('E');
       jMenuItemApplicationsExplorer.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -719,7 +719,7 @@ public class Main
       // Applications/Experimenter
       jMenuItemApplicationsExperimenter = new JMenuItem();
       jMenuApplications.add(jMenuItemApplicationsExperimenter);
-      jMenuItemApplicationsExperimenter.setText("Experimenter");
+      jMenuItemApplicationsExperimenter.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemApplicationsExperimenter_SetText_Text"));
       jMenuItemApplicationsExperimenter.setMnemonic('X');
       jMenuItemApplicationsExperimenter.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -738,7 +738,7 @@ public class Main
       // Applications/KnowledgeFlow
       jMenuItemApplicationsKnowledgeFlow = new JMenuItem();
       jMenuApplications.add(jMenuItemApplicationsKnowledgeFlow);
-      jMenuItemApplicationsKnowledgeFlow.setText("KnowledgeFlow");
+      jMenuItemApplicationsKnowledgeFlow.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemApplicationsKnowledgeFlow_SetText_Text"));
       jMenuItemApplicationsKnowledgeFlow.setMnemonic('K');
       jMenuItemApplicationsKnowledgeFlow.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -758,7 +758,7 @@ public class Main
       // Applications/SimpleCLI
       jMenuItemApplicationsSimpleCLI = new JMenuItem();
       jMenuApplications.add(jMenuItemApplicationsSimpleCLI);
-      jMenuItemApplicationsSimpleCLI.setText("SimpleCLI");
+      jMenuItemApplicationsSimpleCLI.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemApplicationsSimpleCLI_SetText_Text"));
       jMenuItemApplicationsSimpleCLI.setMnemonic('S');
       jMenuItemApplicationsSimpleCLI.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -772,7 +772,7 @@ public class Main
 	    catch (Exception e) {
 	      e.printStackTrace();
 	      JOptionPane.showMessageDialog(
-		  m_Self, "Error instantiating SimpleCLI:\n" + e.getMessage());
+		  m_Self, Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text") + e.getMessage());
 	      return;
 	    }
 	  }
@@ -785,13 +785,13 @@ public class Main
       // Tools
       jMenuTools = new JMenu();
       jMenuBar.add(jMenuTools);
-      jMenuTools.setText("Tools");
+      jMenuTools.setText(Messages.getInstance().getString("Main_InitGUI_JMenuTools_JMenu_SetText_Text")); // ""
       jMenuTools.setMnemonic('T');
 
       // Tools/ArffViewer
       jMenuItemToolsArffViewer = new JMenuItem();
       jMenuTools.add(jMenuItemToolsArffViewer);
-      jMenuItemToolsArffViewer.setText("ArffViewer");
+      jMenuItemToolsArffViewer.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemToolsArffViewer_SetText_Text"));
       jMenuItemToolsArffViewer.setMnemonic('A');
       jMenuItemToolsArffViewer.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -813,7 +813,7 @@ public class Main
       // Tools/SqlViewer
       jMenuItemToolsSqlViewer = new JMenuItem();
       jMenuTools.add(jMenuItemToolsSqlViewer);
-      jMenuItemToolsSqlViewer.setText("SqlViewer");
+      jMenuItemToolsSqlViewer.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemToolsSqlViewer_SetText_Text")); //"SqlViewer"
       jMenuItemToolsSqlViewer.setMnemonic('S');
       jMenuItemToolsSqlViewer.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -852,7 +852,7 @@ public class Main
       // Tools/Bayes net editor
       final JMenuItem jMenuItemBayesNet = new JMenuItem();
       jMenuTools.add(jMenuItemBayesNet);
-      jMenuItemBayesNet.setText("Bayes net editor");
+      jMenuItemBayesNet.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemBayesNet_SetText_Text"));
       jMenuItemBayesNet.setMnemonic('N');
 
       jMenuItemBayesNet.addActionListener(new ActionListener() {
@@ -898,13 +898,13 @@ public class Main
       // Visualization
       jMenuVisualization = new JMenu();
       jMenuBar.add(jMenuVisualization);
-      jMenuVisualization.setText("Visualization");
+      jMenuVisualization.setText(Messages.getInstance().getString("Main_InitGUI_JMenuVisualization_SetText_Text"));
       jMenuVisualization.setMnemonic('V');
 
       // Visualization/Plot
       jMenuItemVisualizationPlot = new JMenuItem();
       jMenuVisualization.add(jMenuItemVisualizationPlot);
-      jMenuItemVisualizationPlot.setText("Plot");
+      jMenuItemVisualizationPlot.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemVisualizationPlot_SetText_Text"));
       jMenuItemVisualizationPlot.setMnemonic('P');
       jMenuItemVisualizationPlot.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -922,7 +922,7 @@ public class Main
 	    if (j > 0)
 	      filenames += ", ";
 	    filenames += filename;
-	    System.err.println("Loading instances from " + filename);
+	    System.err.println(Messages.getInstance().getString("Main_InitGUI_Error_Text") + filename);
 	    try {
 	      Reader r = new java.io.BufferedReader(new FileReader(filename));
 	      Instances i = new Instances(r);
@@ -930,10 +930,10 @@ public class Main
 	      PlotData2D pd1 = new PlotData2D(i);
 
 	      if (j == 0) {
-		pd1.setPlotName("Master plot");
+		pd1.setPlotName(Messages.getInstance().getString("Main_InitGUI_Pd1_SetPlotName_Text_First"));
 		panel.setMasterPlot(pd1);
 	      } else {
-		pd1.setPlotName("Plot "+(j+1));
+		pd1.setPlotName(Messages.getInstance().getString("Main_InitGUI_Pd1_SetPlotName_Text_Second") + (j+1));
 		pd1.m_useCustomColour = true;
 		pd1.m_customColour = (j % 2 == 0) ? Color.red : Color.blue; 
 		panel.addPlot(pd1);
@@ -942,7 +942,7 @@ public class Main
 	    catch (Exception e) {
 	      e.printStackTrace();
 	      JOptionPane.showMessageDialog(
-		  m_Self, "Error loading file '" + files[j] + "':\n" + e.getMessage());
+		  m_Self, Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_First") + files[j] + Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_Second") + e.getMessage());
 	      return;
 	    }
 	  }
@@ -960,7 +960,7 @@ public class Main
       // http://weka.sourceforge.net/wiki/index.php/Visualizing_ROC_curve
       jMenuItemVisualizationROC = new JMenuItem();
       jMenuVisualization.add(jMenuItemVisualizationROC);
-      jMenuItemVisualizationROC.setText("ROC");
+      jMenuItemVisualizationROC.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemVisualizationROC_SetText_Text"));
       jMenuItemVisualizationROC.setMnemonic('R');
       jMenuItemVisualizationROC.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -978,12 +978,12 @@ public class Main
 	  catch (Exception e) {
 	    e.printStackTrace();
 	    JOptionPane.showMessageDialog(
-		m_Self, "Error loading file '" + filename + "':\n" + e.getMessage());
+		m_Self, Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_Third") + filename + Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_Fourth") + e.getMessage());
 	    return;
 	  }
 	  result.setClassIndex(result.numAttributes() - 1);
 	  ThresholdVisualizePanel vmc = new ThresholdVisualizePanel();
-	  vmc.setROCString("(Area under ROC = " + 
+	  vmc.setROCString(Messages.getInstance().getString("Main_InitGUI_Vmc_SetROCString_Text") + 
 	      Utils.doubleToString(ThresholdCurve.getROCArea(result), 4) + ")");
 	  vmc.setName(result.relationName());
 	  PlotData2D tempd = new PlotData2D(result);
@@ -995,7 +995,7 @@ public class Main
 	  catch (Exception e) {
 	    e.printStackTrace();
 	    JOptionPane.showMessageDialog(
-		m_Self, "Error adding plot:\n" + e.getMessage());
+		m_Self, Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_Fifth") + e.getMessage());
 	    return;
 	  }
 
@@ -1009,7 +1009,7 @@ public class Main
       // Visualization/TreeVisualizer
       jMenuItemVisualizationTreeVisualizer = new JMenuItem();
       jMenuVisualization.add(jMenuItemVisualizationTreeVisualizer);
-      jMenuItemVisualizationTreeVisualizer.setText("TreeVisualizer");
+      jMenuItemVisualizationTreeVisualizer.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemVisualizationTreeVisualizer_SetText_Text"));
       jMenuItemVisualizationTreeVisualizer.setMnemonic('T');
       jMenuItemVisualizationTreeVisualizer.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -1029,7 +1029,7 @@ public class Main
 	  catch (Exception e) {
 	    e.printStackTrace();
 	    JOptionPane.showMessageDialog(
-		m_Self, "Error loading file '" + filename + "':\n" + e.getMessage());
+		m_Self, Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_Sixth") + filename + Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_Seventh") + e.getMessage());
 	    return;
 	  }
 
@@ -1044,7 +1044,7 @@ public class Main
       // Visualization/GraphVisualizer
       jMenuItemVisualizationGraphVisualizer = new JMenuItem();
       jMenuVisualization.add(jMenuItemVisualizationGraphVisualizer);
-      jMenuItemVisualizationGraphVisualizer.setText("GraphVisualizer");
+      jMenuItemVisualizationGraphVisualizer.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemVisualizationGraphVisualizer_SetText_Text"));
       jMenuItemVisualizationGraphVisualizer.setMnemonic('G');
       jMenuItemVisualizationGraphVisualizer.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -1068,7 +1068,7 @@ public class Main
 	  catch (Exception e) {
 	    e.printStackTrace();
 	    JOptionPane.showMessageDialog(
-		m_Self, "Error loading file '" + filename + "':\n" + e.getMessage());
+		m_Self, Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_Eighth") + filename + Messages.getInstance().getString("Main_InitGUI_Exception_JOptionPaneShowMessageDialog_Text_nineth") + e.getMessage());
 	    return;
 	  }
 
@@ -1083,7 +1083,7 @@ public class Main
       // Visualization/BoundaryVisualizer
       jMenuItemVisualizationBoundaryVisualizer = new JMenuItem();
       jMenuVisualization.add(jMenuItemVisualizationBoundaryVisualizer);
-      jMenuItemVisualizationBoundaryVisualizer.setText("BoundaryVisualizer");
+      jMenuItemVisualizationBoundaryVisualizer.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemVisualizationBoundaryVisualizer_SetText_Text"));
       jMenuItemVisualizationBoundaryVisualizer.setMnemonic('B');
       jMenuItemVisualizationBoundaryVisualizer.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -1102,7 +1102,7 @@ public class Main
       });
 
       // Extensions
-      jMenuExtensions = new JMenu("Extensions");
+      jMenuExtensions = new JMenu(Messages.getInstance().getString("Main_InitGUI_JMenuExtensions_JMenu_Text"));
       jMenuExtensions.setMnemonic(java.awt.event.KeyEvent.VK_E);
       jMenuBar.add(jMenuExtensions);
       jMenuExtensions.setVisible(false);
@@ -1168,7 +1168,7 @@ public class Main
       }
 
       // Windows
-      jMenuWindows = new JMenu("Windows");
+      jMenuWindows = new JMenu(Messages.getInstance().getString("Main_InitGUI_JMenuWindows_JMenu_Text"));
       jMenuWindows.setMnemonic(java.awt.event.KeyEvent.VK_W);
       jMenuBar.add(jMenuWindows);
       jMenuWindows.setVisible(false);  // initially, there are no windows open
@@ -1176,17 +1176,17 @@ public class Main
       // Help
       jMenuHelp = new JMenu();
       jMenuBar.add(jMenuHelp);
-      jMenuHelp.setText("Help");
+      jMenuHelp.setText(Messages.getInstance().getString("Main_InitGUI_JMenuHelp_SetText_Text"));
       jMenuHelp.setMnemonic('H');
 
       // Help/Homepage
       jMenuItemHelpHomepage = new JMenuItem();
       jMenuHelp.add(jMenuItemHelpHomepage);
-      jMenuItemHelpHomepage.setText("Weka homepage");
+      jMenuItemHelpHomepage.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemHelpHomepage_SetText_Text"));
       jMenuItemHelpHomepage.setMnemonic('H');
       jMenuItemHelpHomepage.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
-	  BrowserHelper.openURL(m_Self, "http://www.cs.waikato.ac.nz/~ml/weka/");
+	  BrowserHelper.openURL(m_Self, Messages.getInstance().getString("Main_InitGUI_BrowserHelper_OpenURL_Text_First"));
 	}
       });
 
@@ -1206,22 +1206,22 @@ public class Main
       // Help/WekaWiki
       jMenuItemHelpWekaWiki = new JMenuItem();
       jMenuHelp.add(jMenuItemHelpWekaWiki);
-      jMenuItemHelpWekaWiki.setText("HOWTOs, code snippets, etc.");
+      jMenuItemHelpWekaWiki.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemHelpWekaWiki_SetText_Text"));
       jMenuItemHelpWekaWiki.setMnemonic('W');
       jMenuItemHelpWekaWiki.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
-	  BrowserHelper.openURL(m_Self, "http://weka.wiki.sourceforge.net/");
+	  BrowserHelper.openURL(m_Self, Messages.getInstance().getString("Main_InitGUI_BrowserHelper_OpenURL_Text_Second"));
 	}
       });
 
       // Help/Sourceforge
       jMenuItemHelpSourceforge = new JMenuItem();
       jMenuHelp.add(jMenuItemHelpSourceforge);
-      jMenuItemHelpSourceforge.setText("Weka on SourceForge");
+      jMenuItemHelpSourceforge.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemHelpSourceforge_SetText_Text"));
       jMenuItemHelpSourceforge.setMnemonic('F');
       jMenuItemHelpSourceforge.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
-	  BrowserHelper.openURL(m_Self, "http://sourceforge.net/projects/weka/");
+	  BrowserHelper.openURL(m_Self, Messages.getInstance().getString("Main_InitGUI_BrowserHelperOpenURL_Text_Third"));
 	}
       });
 
@@ -1230,7 +1230,7 @@ public class Main
       // Help/SystemInfo
       jMenuItemHelpSystemInfo = new JMenuItem();
       jMenuHelp.add(jMenuItemHelpSystemInfo);
-      jMenuItemHelpSystemInfo.setText("SystemInfo");
+      jMenuItemHelpSystemInfo.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemHelpSystemInfo_SetText_Text"));
       jMenuItemHelpHomepage.setMnemonic('S');
       jMenuItemHelpSystemInfo.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
@@ -1252,7 +1252,7 @@ public class Main
 	      data[i][0] = names.get(i).toString();
 	      data[i][1] = info.get(data[i][0]).toString();
 	    }
-	    String[] titles = new String[]{"Key", "Value"};
+	    String[] titles = new String[]{Messages.getInstance().getString("Main_InitGUI_Title_Text_First"), Messages.getInstance().getString("Main_InitGUI_Title_Text_Second")};
 	    JTable table = new JTable(data, titles);
 
 	    createFrame(
@@ -1270,14 +1270,14 @@ public class Main
       // Help/About
       jMenuItemHelpAbout = new JMenuItem();
       jMenuHelp.add(jMenuItemHelpAbout);
-      jMenuItemHelpAbout.setText("About");
+      jMenuItemHelpAbout.setText(Messages.getInstance().getString("Main_InitGUI_JMenuItemHelpAbout_SetText_Text"));
       jMenuItemHelpAbout.setMnemonic('A');
       jMenuItemHelpAbout.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent evt) {
 	  String title = jMenuItemHelpAbout.getText();
 	  if (!containsWindow(title)) {
 	    JPanel wekaPan = new JPanel();
-	    wekaPan.setToolTipText("Weka, a native bird of New Zealand");
+	    wekaPan.setToolTipText(Messages.getInstance().getString("Main_InitGUI_WekaPan_JPanel_SetToolTipText_Text"));
 	    ImageIcon wii = new ImageIcon(Toolkit.getDefaultToolkit().getImage(ClassLoader.getSystemResource("weka/gui/weka3.gif")));
 	    JLabel wekaLab = new JLabel(wii);
 	    wekaPan.add(wekaLab);
@@ -1288,12 +1288,12 @@ public class Main
 	    JPanel titlePan = new JPanel();
 	    titlePan.setLayout(new GridLayout(8,1));
 	    titlePan.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
-	    titlePan.add(new JLabel("Waikato Environment for", SwingConstants.CENTER));
-	    titlePan.add(new JLabel("Knowledge Analysis", SwingConstants.CENTER));
-	    titlePan.add(new JLabel(""));
-	    titlePan.add(new JLabel("Version " + Version.VERSION, SwingConstants.CENTER));
-	    titlePan.add(new JLabel(""));
-	    titlePan.add(new JLabel("(c) " + Copyright.getFromYear() + " - " + Copyright.getToYear(), SwingConstants.CENTER));
+	    titlePan.add(new JLabel(Messages.getInstance().getString("Main_InitGUI_TitlePan_Add_JLabel_Text_First"), SwingConstants.CENTER));
+	    titlePan.add(new JLabel(Messages.getInstance().getString("Main_InitGUI_TitlePan_Add_JLabel_Text_Second"), SwingConstants.CENTER));
+	    titlePan.add(new JLabel(Messages.getInstance().getString("Main_InitGUI_TitlePan_Add_JLabel_Text_Third")));
+	    titlePan.add(new JLabel(Messages.getInstance().getString("Main_InitGUI_TitlePan_Add_JLabel_Text_Fourth") + Version.VERSION, SwingConstants.CENTER));
+	    titlePan.add(new JLabel(Messages.getInstance().getString("Main_InitGUI_TitlePan_Add_JLabel_Text_Fifth")));
+	    titlePan.add(new JLabel(Messages.getInstance().getString("Main_InitGUI_TitlePan_Add_JLabel_Text_Sixth") + Copyright.getFromYear() + " - " + Copyright.getToYear(), SwingConstants.CENTER));
 	    titlePan.add(new JLabel(Copyright.getOwner(), SwingConstants.CENTER));
 	    titlePan.add(new JLabel(Copyright.getAddress(), SwingConstants.CENTER));
 
@@ -1344,7 +1344,7 @@ public class Main
   protected void createTitle(String title) {
     String	newTitle;
     
-    newTitle = "Weka " + new Version();
+    newTitle = Messages.getInstance().getString("Main_CreateTitle_NewTitle_Text") + new Version();
     if (title.length() != 0)
       newTitle += " - " + title;
     
@@ -1576,7 +1576,7 @@ public class Main
     jMenuWindows.removeAll();
     
     // minimize + restore + separator
-    menuItem = new JMenuItem("Minimize");
+    menuItem = new JMenuItem(Messages.getInstance().getString("Main_CreateWindowMenu_MenuItem_JMenuItem_Text_First"));
     menuItem.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent evt) {
         minimizeWindows();
@@ -1584,7 +1584,7 @@ public class Main
     });
     jMenuWindows.add(menuItem);
     
-    menuItem = new JMenuItem("Restore");
+    menuItem = new JMenuItem(Messages.getInstance().getString("Main_CreateWindowMenu_MenuItem_JMenuItem_Text_Second"));
     menuItem.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent evt) {
         restoreWindows();
@@ -1698,9 +1698,9 @@ public class Main
       	       + "\n";
     }
     result.addElement(new Option(
-	"\tDetermines the layout of the GUI:\n"
+    		Messages.getInstance().getString("Main_ListOptions_Option_Text_First")
 	+ desc
-	+ "\t(default: " + new SelectedTag(GUI_MDI, TAGS_GUI) + ")",
+	+ Messages.getInstance().getString("Main_ListOptions_Option_Text_Second") + new SelectedTag(GUI_MDI, TAGS_GUI) + ")",
 	"gui", 1, "-gui " + Tag.toOptionList(TAGS_GUI)));
 
     return result.elements();
@@ -1776,7 +1776,7 @@ public class Main
    * @param args	the commandline arguments - ignored
    */
   public static void main(String[] args) {
-    weka.core.logging.Logger.log(weka.core.logging.Logger.Level.INFO, "Logging started");
+    weka.core.logging.Logger.log(weka.core.logging.Logger.Level.INFO, Messages.getInstance().getString("Main_Main_Logger_Text"));
     
     LookAndFeel.setLookAndFeel();
     
@@ -1787,12 +1787,12 @@ public class Main
       // help?
       if (Utils.getFlag('h', args)) {
 	System.out.println();
-	System.out.println("Help requested.");
+	System.out.println(Messages.getInstance().getString("Main_Main_Text_First"));
 	System.out.println();
-	System.out.println("General options:");
+	System.out.println(Messages.getInstance().getString("Main_Main_Text_Second"));
 	System.out.println();
 	System.out.println("-h");
-	System.out.println("\tprints this help screen");
+	System.out.println(Messages.getInstance().getString("Main_Main_Text_Third"));
 	System.out.println();
 
 	Enumeration enu = new Main().listOptions();
@@ -1846,9 +1846,9 @@ public class Main
 		m_Memory.stopThreads();
 		
 		// display error
-		System.err.println("\ndisplayed message:");
+		System.err.println(Messages.getInstance().getString("Main_Main_Thread_Run_Error_Text_First"));
 		m_Memory.showOutOfMemory();
-		System.err.println("\nexiting");
+		System.err.println(Messages.getInstance().getString("Main_Main_Thread_Run_Error_Text_Second"));
 		System.exit(-1);
 	      }
 	      
