@@ -22,8 +22,14 @@
 
 package weka.gui.beans;
 
+import java.io.IOException;
+import java.io.ObjectInputStream;
+
+import weka.core.Environment;
+import weka.core.EnvironmentHandler;
 import weka.core.Instances;
 import weka.core.OptionHandler;
+import weka.core.SerializedObject;
 import weka.core.Utils;
 import weka.core.converters.ArffSaver;
 import weka.core.converters.DatabaseConverter;
@@ -33,12 +39,12 @@ import weka.core.converters.DatabaseSaver;
  * Saves data sets using weka.core.converter classes
  *
  * @author <a href="mailto:mutter@cs.waikato.ac.nz">Stefan Mutter</a>
- * @version $Revision: 4836 $
+ * @version $Revision: 7710 $
  *
  */
 public class Saver
   extends AbstractDataSink
-  implements WekaWrapper {
+  implements WekaWrapper, EnvironmentHandler {
 
   /** for serialization */
   private static final long serialVersionUID = 5371716690308950755L;
@@ -67,6 +73,7 @@ public class Saver
    * Saver
    */
   private weka.core.converters.Saver m_Saver= new ArffSaver();
+  private weka.core.converters.Saver m_SaverTemplate = m_Saver;
 
   /**
    * The relation name that becomes part of the file name
@@ -89,6 +96,15 @@ public class Saver
    */
   private int m_count;
   
+  /**
+   * The environment variables.
+   */
+  protected transient Environment m_env;
+  
+  private weka.core.converters.Saver makeCopy() throws Exception {
+    return (weka.core.converters.Saver)new SerializedObject(m_SaverTemplate).getObject();
+  }
+  
   private class SaveBatchThread extends Thread {
     private DataSink m_DS;
 
@@ -99,35 +115,36 @@ public class Saver
     public void run() {
       try {
         m_visual.setAnimated();
+                
         m_Saver.setInstances(m_dataSet);
         if (m_logger != null) {
-          m_logger.statusMessage(statusMessagePrefix() + "Saving "
-              + m_dataSet.relationName() + "...");
+          m_logger.statusMessage(statusMessagePrefix() + Messages.getInstance().getString("Saver_SaveBatchThread_Run_StatusMessage_Text_First")
+              + m_dataSet.relationName() + Messages.getInstance().getString("Saver_SaveBatchThread_Run_StatusMessage_Text_Second"));
         }
         m_Saver.writeBatch();
         if (m_logger != null) {
-          m_logger.logMessage("[Saver] " + statusMessagePrefix() 
-              + "Save successful.");
+          m_logger.logMessage(Messages.getInstance().getString("Saver_SaveBatchThread_Run_LogMessage_Text_First") + statusMessagePrefix() 
+              + Messages.getInstance().getString("Saver_SaveBatchThread_Run_LogMessage_Text_Second"));
         }
 	
       } catch (Exception ex) {
         if (m_logger != null) {
           m_logger.statusMessage(statusMessagePrefix()
-              + "ERROR (See log for details)");
-          m_logger.logMessage("[Saver] " + statusMessagePrefix()
-              + " problem saving. " 
+              + Messages.getInstance().getString("Saver_SaveBatchThread_Run_StatusMessage_Text_Third"));
+          m_logger.logMessage(Messages.getInstance().getString("Saver_SaveBatchThread_Run_LogMessage_Text_Third") + statusMessagePrefix()
+              + Messages.getInstance().getString("Saver_SaveBatchThread_Run_LogMessage_Text_Fourth") 
               + ex.getMessage());
         }
 	ex.printStackTrace();
       } finally {
         if (Thread.currentThread().isInterrupted()) {
           if (m_logger != null) {
-            m_logger.logMessage("[Saver] " + statusMessagePrefix()
-                + " Saving interrupted!!");
+            m_logger.logMessage(Messages.getInstance().getString("Saver_SaveBatchThread_Run_LogMessage_Text_Fifth") + statusMessagePrefix()
+                + Messages.getInstance().getString("Saver_SaveBatchThread_Run_LogMessage_Text_Sixth"));
           }
         }
         if (m_logger != null) {
-          m_logger.statusMessage(statusMessagePrefix() + "Done.");
+          m_logger.statusMessage(statusMessagePrefix() + Messages.getInstance().getString("Saver_SaveBatchThread_Run_StatusMessage_Text_Fourth"));
         }
         block(false);
 	m_visual.setStatic();
@@ -179,7 +196,7 @@ public class Saver
   /** Contsructor */  
   public Saver() {
     super();
-    setSaver(m_Saver);
+    setSaverTemplate(m_Saver);
     m_fileName = "";
     m_dataSet = null;
     m_count = 0;
@@ -203,17 +220,37 @@ public class Saver
   public String getCustomName() {
     return m_visual.getText();
   }  
+  
+  /**
+   * Set environment variables to use.
+   * 
+   * @param env the environment variables to
+   * use
+   */
+  public void setEnvironment(Environment env) {
+    m_env = env;
+  }
+  
+  /**
+   * Pass the environment variables on the the wrapped saver
+   */
+  private void passEnvOnToSaver() {
+    // set environment variables
+    if (m_SaverTemplate instanceof EnvironmentHandler && m_env != null) {
+      ((EnvironmentHandler)m_Saver).setEnvironment(m_env);
+    }
+  }
 
   /** Set the loader to use
    * @param saver a Saver
    */
-  public void setSaver(weka.core.converters.Saver saver) {
+  public void setSaverTemplate(weka.core.converters.Saver saver) {
     boolean loadImages = true;
     if (saver.getClass().getName().
-	compareTo(m_Saver.getClass().getName()) == 0) {
+	compareTo(m_SaverTemplate.getClass().getName()) == 0) {
       loadImages = false;
     }
-    m_Saver = saver;
+    m_SaverTemplate = saver;
     String saverName = saver.getClass().toString();
     saverName = saverName.substring(saverName.
 				      lastIndexOf('.')+1, 
@@ -229,8 +266,8 @@ public class Saver
 
     
     // get global info
-    m_globalInfo = KnowledgeFlowApp.getGlobalInfo(m_Saver);
-    if(m_Saver instanceof DatabaseConverter)
+    m_globalInfo = KnowledgeFlowApp.getGlobalInfo(m_SaverTemplate);
+    if(m_SaverTemplate instanceof DatabaseConverter)
         m_isDBSaver = true;
     else
         m_isDBSaver = false;
@@ -276,9 +313,21 @@ public class Saver
    */  
   public synchronized void acceptDataSet(DataSetEvent e) {
   
+      try {
+        m_Saver = makeCopy();
+      } catch (Exception ex) {
+        if (m_logger != null) {
+          m_logger.statusMessage(statusMessagePrefix()
+              + Messages.getInstance().getString("Saver_AcceptDataSet_StatusMessage_Text_First"));
+          m_logger.logMessage(Messages.getInstance().getString("Saver_AcceptDataSet_LogMessage_Text_First") + statusMessagePrefix()
+              + Messages.getInstance().getString("Saver_AcceptDataSet_LogMessage_Text_Second")
+              + ex.getMessage());
+        }
+      }
+      passEnvOnToSaver();
       m_fileName = sanitizeFilename(e.getDataSet().relationName());
       m_dataSet = e.getDataSet();
-      if(e.isStructureOnly() && m_isDBSaver && ((DatabaseSaver)m_Saver).getRelationForTableName()){//
+      if(e.isStructureOnly() && m_isDBSaver && ((DatabaseSaver)m_SaverTemplate).getRelationForTableName()){//
           ((DatabaseSaver)m_Saver).setTableName(m_fileName);
       }
       if(!e.isStructureOnly()){
@@ -290,18 +339,70 @@ public class Saver
             }
           }
           saveBatch();
-          System.out.println("...relation "+ m_fileName +" saved.");
+          System.out.println(Messages.getInstance().getString("Saver_AcceptDataSet_Text_First")+ m_fileName + Messages.getInstance().getString("Saver_AcceptDataSet_Text_Second"));
       }
   }
   
+  /**
+   * Method reacts to a threshold data event ans starts the writing process
+   * in batch mode.
+   * 
+   * @param e threshold data event.
+   */
+  public synchronized void acceptDataSet(ThresholdDataEvent e) {
+    try {
+      m_Saver = makeCopy();
+    } catch (Exception ex) {
+      if (m_logger != null) {
+        m_logger.statusMessage(statusMessagePrefix()
+            + Messages.getInstance().getString("Saver_AcceptDataSet_StatusMessage_Text_Second"));
+        m_logger.logMessage(Messages.getInstance().getString("Saver_AcceptDataSet_LogMessage_Text_Third") + statusMessagePrefix()
+            + Messages.getInstance().getString("Saver_AcceptDataSet_LogMessage_Text_Fourth")
+            + ex.getMessage());
+      }
+    }
+    
+    passEnvOnToSaver();
+    m_fileName = sanitizeFilename(e.getDataSet().getPlotInstances().relationName());
+    m_dataSet = e.getDataSet().getPlotInstances();
+    
+    if(m_isDBSaver && ((DatabaseSaver)m_SaverTemplate).getRelationForTableName()){//
+      ((DatabaseSaver)m_Saver).setTableName(m_fileName);
+      ((DatabaseSaver)m_Saver).setRelationForTableName(false);
+    }
+
+    if(!m_isDBSaver){
+      try{
+        m_Saver.setDirAndPrefix(m_fileName,"");
+      }catch (Exception ex){
+        System.out.println(ex);
+      }
+    }
+    saveBatch();
+    System.out.println(Messages.getInstance().getString("Saver_AcceptDataSet_Text_Third") + m_fileName + Messages.getInstance().getString("Saver_AcceptDataSet_Text_Fourth"));
+  }
+  
   /** Method reacts to a test set event and starts the writing process in batch mode
    * @param e test set event
    */  
   public synchronized void acceptTestSet(TestSetEvent e) {
-  
+      if (e.isStructureOnly()) {
+        try {
+          m_Saver = makeCopy();
+        } catch (Exception ex) {
+          if (m_logger != null) {
+            m_logger.statusMessage(statusMessagePrefix()
+                + Messages.getInstance().getString("Saver_AcceptTestSet_StatusMessage_Text_First"));
+            m_logger.logMessage(Messages.getInstance().getString("Saver_AcceptTestSet_LogMessage_Text_First") + statusMessagePrefix()
+                + Messages.getInstance().getString("Saver_AcceptTestSet_LogMessage_Text_Second")
+                + ex.getMessage());
+          }
+        }
+      }
+      passEnvOnToSaver();
       m_fileName = sanitizeFilename(e.getTestSet().relationName());
       m_dataSet = e.getTestSet();
-      if(e.isStructureOnly() && m_isDBSaver && ((DatabaseSaver)m_Saver).getRelationForTableName()){
+      if(e.isStructureOnly() && m_isDBSaver && ((DatabaseSaver)m_SaverTemplate).getRelationForTableName()){
           ((DatabaseSaver)m_Saver).setTableName(m_fileName);
       }
       if(!e.isStructureOnly()){
@@ -313,12 +414,15 @@ public class Saver
             }
           }
           else{
+              ((DatabaseSaver)m_Saver).setRelationForTableName(false);
               String setName = ((DatabaseSaver)m_Saver).getTableName();
               setName = setName.replaceFirst("_[tT][eE][sS][tT]_[0-9]+_[oO][fF]_[0-9]+","");
               ((DatabaseSaver)m_Saver).setTableName(setName+"_test_"+e.getSetNumber()+"_of_"+e.getMaxSetNumber());
           }
           saveBatch();
-          System.out.println("... test set "+e.getSetNumber()+" of "+e.getMaxSetNumber()+" for relation "+ m_fileName +" saved.");
+          System.out.println(Messages.getInstance().getString("Saver_AcceptTestSet_Text_First") + e.getSetNumber() + 
+        		  Messages.getInstance().getString("Saver_AcceptTestSet_Text_Second") + e.getMaxSetNumber() + 
+        		  Messages.getInstance().getString("Saver_AcceptTestSet_Text_Third") + m_fileName + Messages.getInstance().getString("Saver_AcceptTestSet_Text_Fourth"));
       }
   }
   
@@ -327,10 +431,24 @@ public class Saver
    * @param e a training set event
    */  
   public synchronized void acceptTrainingSet(TrainingSetEvent e) {
+    if (e.isStructureOnly()) {
+      try {
+        m_Saver = makeCopy();
+      } catch (Exception ex) {
+        if (m_logger != null) {
+          m_logger.statusMessage(statusMessagePrefix()
+              + Messages.getInstance().getString("Saver_AcceptTrainingSet_StatusMessage_Text_First"));
+          m_logger.logMessage(Messages.getInstance().getString("Saver_AcceptTrainingSet_LogMessage_Text_First") + statusMessagePrefix()
+              + Messages.getInstance().getString("Saver_AcceptTrainingSet_LogMessage_Text_Second")
+              + ex.getMessage());
+        }
+      }
+    }
   
+      passEnvOnToSaver();
       m_fileName = sanitizeFilename(e.getTrainingSet().relationName());
       m_dataSet = e.getTrainingSet();
-      if(e.isStructureOnly() && m_isDBSaver && ((DatabaseSaver)m_Saver).getRelationForTableName()){
+      if(e.isStructureOnly() && m_isDBSaver && ((DatabaseSaver)m_SaverTemplate).getRelationForTableName()){
            ((DatabaseSaver)m_Saver).setTableName(m_fileName);
       }
       if(!e.isStructureOnly()){
@@ -342,12 +460,13 @@ public class Saver
             }
           }
           else{
+            ((DatabaseSaver)m_Saver).setRelationForTableName(false);
               String setName = ((DatabaseSaver)m_Saver).getTableName();
               setName = setName.replaceFirst("_[tT][rR][aA][iI][nN][iI][nN][gG]_[0-9]+_[oO][fF]_[0-9]+","");
               ((DatabaseSaver)m_Saver).setTableName(setName+"_training_"+e.getSetNumber()+"_of_"+e.getMaxSetNumber());
           }
           saveBatch();
-          System.out.println("... training set "+e.getSetNumber()+" of "+e.getMaxSetNumber()+" for relation "+ m_fileName +" saved.");
+          System.out.println(Messages.getInstance().getString("Saver_AcceptTrainingSet_Text_First") + e.getSetNumber() + Messages.getInstance().getString("Saver_AcceptTrainingSet_Text_Second") + e.getMaxSetNumber()+ Messages.getInstance().getString("Saver_AcceptTrainingSet_Text_Third") + m_fileName + Messages.getInstance().getString("Saver_AcceptTrainingSet_Text_Fourth"));
       }
   }
   
@@ -355,12 +474,12 @@ public class Saver
   public synchronized void saveBatch(){
   
       m_Saver.setRetrieval(m_Saver.BATCH);
-      String visText = this.getName();
+/*      String visText = this.getName();
       try {
         visText = (m_fileName.length() > 0) ? m_fileName : m_Saver.filePrefix();
       } catch (Exception ex) {        
       }
-      m_visual.setText(visText);
+      m_visual.setText(visText); */
       m_ioThread = new SaveBatchThread(Saver.this);
       m_ioThread.setPriority(Thread.MIN_PRIORITY);
       m_ioThread.start();
@@ -376,17 +495,32 @@ public class Saver
       
       
       if(e.getStatus() == e.FORMAT_AVAILABLE){
+        // start of a new stream
+        try {
+          m_Saver = makeCopy();
+        } catch (Exception ex) {
+          if (m_logger != null) {
+            m_logger.statusMessage(statusMessagePrefix()
+                + Messages.getInstance().getString("Saver_AcceptInstance_StatusMessage_Text_First"));
+            m_logger.logMessage(Messages.getInstance().getString("Saver_AcceptInstance_LogMessage_Text_First") + statusMessagePrefix()
+                + Messages.getInstance().getString("Saver_AcceptInstance_LogMessage_Text_Second")
+                + ex.getMessage());
+          }
+        }
         m_Saver.setRetrieval(m_Saver.INCREMENTAL);
         m_structure = e.getStructure();
         m_fileName = sanitizeFilename(m_structure.relationName());
         m_Saver.setInstances(m_structure);
         if(m_isDBSaver)
-            if(((DatabaseSaver)m_Saver).getRelationForTableName())
+            if(((DatabaseSaver)m_SaverTemplate).getRelationForTableName()) {
                 ((DatabaseSaver)m_Saver).setTableName(m_fileName);
+                ((DatabaseSaver)m_Saver).setRelationForTableName(false);
+            }
       }
       if(e.getStatus() == e.INSTANCE_AVAILABLE){
         m_visual.setAnimated();
         if(m_count == 0){
+            passEnvOnToSaver();
             if(!m_isDBSaver){
                 try{
                     m_Saver.setDirAndPrefix(m_fileName,"");
@@ -398,13 +532,14 @@ public class Saver
             m_count ++;
         }
         try{
-          String visText = this.getName();
+/*          String visText = this.getName();
           visText = (m_fileName.length() > 0) ? m_fileName : m_Saver.filePrefix();
-            m_visual.setText(m_fileName);
+            m_visual.setText(m_fileName); */
             m_Saver.writeIncremental(e.getInstance());
         } catch (Exception ex) {
             m_visual.setStatic();
-            System.err.println("Instance "+e.getInstance() +" could not been saved");
+            System.err.println(Messages.getInstance().getString("Saver_AcceptInstance_Error_Text_First") + e.getInstance() + 
+            		Messages.getInstance().getString("Saver_AcceptInstance_Error_Text_Second"));
             ex.printStackTrace();
         }
       }
@@ -414,14 +549,14 @@ public class Saver
             m_Saver.writeIncremental(null);
             //m_firstNotice = true;
             m_visual.setStatic();
-            System.out.println("...relation "+ m_fileName +" saved.");
-            String visText = this.getName();
+            System.out.println(Messages.getInstance().getString("Saver_AcceptInstance_Text_First") + m_fileName + Messages.getInstance().getString("Saver_AcceptInstance_Text_Second"));
+/*            String visText = this.getName();
             visText = (m_fileName.length() > 0) ? m_fileName : m_Saver.filePrefix();
-            m_visual.setText(visText);     
+            m_visual.setText(visText); */     
             m_count = 0;
         } catch (Exception ex) {
             m_visual.setStatic();
-            System.err.println("File could not have been closed.");
+            System.err.println(Messages.getInstance().getString("Saver_AcceptInstance_Text_Third"));
             ex.printStackTrace();
         }
       }
@@ -434,8 +569,8 @@ public class Saver
    *
    * @return a <code>weka.core.converters.Saver</code> value
    */
-  public weka.core.converters.Saver getSaver() {
-    return m_Saver;
+  public weka.core.converters.Saver getSaverTemplate() {
+    return m_SaverTemplate;
   }
 
   /**
@@ -445,12 +580,10 @@ public class Saver
    */
   public void setWrappedAlgorithm(Object algorithm) 
     {
-
     if (!(algorithm instanceof weka.core.converters.Saver)) { 
-      throw new IllegalArgumentException(algorithm.getClass()+" : incorrect "
-					 +"type of algorithm (Loader)");
+      throw new IllegalArgumentException(algorithm.getClass()+  Messages.getInstance().getString("Saver_SetWrappedAlgorithm_IllegalArgumentException_Text"));
     }
-    setSaver((weka.core.converters.Saver)algorithm);
+    setSaverTemplate((weka.core.converters.Saver)algorithm);
   }
 
   /**
@@ -459,7 +592,7 @@ public class Saver
    * @return a Saver
    */
   public Object getWrappedAlgorithm() {
-    return getSaver();
+    return getSaverTemplate();
   }
   
   /**
@@ -504,6 +637,16 @@ public class Saver
             : "");
   }
   
+  // Custom de-serialization in order to set default
+  // environment variables on de-serialization
+  private void readObject(ObjectInputStream aStream) 
+    throws IOException, ClassNotFoundException {
+    aStream.defaultReadObject();
+    
+    // set a default environment to use
+    m_env = Environment.getSystemWide();
+  }
+  
   
   /** The main method for testing
    * @param args
