@@ -16,12 +16,13 @@
 
 /*
  *    SerializedModelSaver.java
- *    Copyright (C) 2008 Pentaho Corporation
+ *    Copyright (C) 2008 University of Waikato, Hamilton, New Zealand
  *
  */
 
 package weka.gui.beans;
 
+import java.io.ObjectInputStream;
 import java.io.Serializable;
 import java.io.File;
 import java.io.ObjectOutputStream;
@@ -37,6 +38,7 @@ import javax.swing.JPanel;
 import weka.classifiers.Classifier;
 import weka.core.Instances;
 import weka.core.Environment;
+import weka.core.EnvironmentHandler;
 import weka.core.xml.KOML;
 import weka.core.xml.XStream;
 import weka.core.Tag;
@@ -46,13 +48,13 @@ import weka.core.Utils;
  * A bean that saves serialized models
  *
  * @author Mark Hall (mhall{[at]}pentaho{[dot]}org
- * @version $Revision: 4813 $
+ * @version $Revision: 7059 $
  */
 public class SerializedModelSaver
   extends JPanel
   implements BeanCommon, Visible, BatchClassifierListener, 
              IncrementalClassifierListener, BatchClustererListener,
-	     Serializable {
+	     EnvironmentHandler, Serializable {
 
   /** for serialization */
   private static final long serialVersionUID = 3956528599473814287L;
@@ -109,20 +111,25 @@ public class SerializedModelSaver
   public static ArrayList<Tag> s_fileFormatsAvailable;
   static {
     s_fileFormatsAvailable = new ArrayList<Tag>();
-    s_fileFormatsAvailable.add(new Tag(BINARY, "Binary serialized model file (*"
-                                       + FILE_EXTENSION + ")", "", false));
+    s_fileFormatsAvailable.add(new Tag(BINARY, Messages.getInstance().getString("SerializedModelSaver_FileFormatsAvailable_Text_First")
+                                       + FILE_EXTENSION + Messages.getInstance().getString("SerializedModelSaver_FileFormatsAvailable_Text_Second"), "", false));
     if (KOML.isPresent()) {
       s_fileFormatsAvailable.add(new Tag(KOMLV,
-                                         "XML serialized model file (*"
-                                         + KOML.FILE_EXTENSION + FILE_EXTENSION + ")", "", false));
+    		  Messages.getInstance().getString("SerializedModelSaver_FileFormatsAvailable_Text_Third")
+                                         + KOML.FILE_EXTENSION + FILE_EXTENSION + Messages.getInstance().getString("SerializedModelSaver_FileFormatsAvailable_Text_Fourth"), "", false));
     }
 
     if (XStream.isPresent()) {
       s_fileFormatsAvailable.add(new Tag(XSTREAM,
-                                         "XML serialized model file (*"
-                                         + XStream.FILE_EXTENSION + FILE_EXTENSION + ")", "", false));
+    		  Messages.getInstance().getString("SerializedModelSaver_FileFormatsAvailable_Text_Fifth")
+                                         + XStream.FILE_EXTENSION + FILE_EXTENSION + Messages.getInstance().getString("SerializedModelSaver_FileFormatsAvailable_Text_Sixth"), "", false));
     }
   }
+  
+  /**
+   * The environment variables.
+   */
+  protected transient Environment m_env;
 
   /**
    * Constructor.
@@ -132,6 +139,8 @@ public class SerializedModelSaver
     setLayout(new BorderLayout());
     add(m_visual, BorderLayout.CENTER);
     m_fileFormat = s_fileFormatsAvailable.get(0);
+    
+    m_env = Environment.getSystemWide();
   }
 
   /**
@@ -293,17 +302,17 @@ public class SerializedModelSaver
 
     String prefix = "";
     try {
-      prefix = Environment.substitute(m_filenamePrefix);
+      prefix = m_env.substitute(m_filenamePrefix);
     } catch (Exception ex) {
       stop(); // stop all processing
-      String message = "[SerializedModelSaver] " 
+      String message = Messages.getInstance().getString("SerializedModelSaver_AcceptClusterer_Message_Text_First") 
         + statusMessagePrefix() 
-        + " Can't save model. Reason: " 
+        + Messages.getInstance().getString("SerializedModelSaver_AcceptClusterer_Message_Text_Second")
         + ex.getMessage();
       if (m_logger != null) {
         m_logger.logMessage(message);
         m_logger.statusMessage(statusMessagePrefix()
-            + "ERROR (See log for details)");
+            + Messages.getInstance().getString("SerializedModelSaver_AcceptClusterer_StatusMessage_Text_First"));
       } else {
         System.err.println(message);
       }
@@ -319,16 +328,16 @@ public class SerializedModelSaver
     
     String dirName = m_directory.getPath();
     try {
-      dirName = Environment.substitute(dirName);
+      dirName = m_env.substitute(dirName);
     } catch (Exception ex) {
       stop(); // stop all processing
-      String message = "[SerializedModelSaver] "
-        + statusMessagePrefix() + " Can't save model. Reason: " 
+      String message = Messages.getInstance().getString("SerializedModelSaver_AcceptClusterer_Message_Text_Third")
+        + statusMessagePrefix() + Messages.getInstance().getString("SerializedModelSaver_AcceptClusterer_Message_Text_Fourth") 
                            + ex.getMessage();
       if (m_logger != null) {
         m_logger.logMessage(message);
         m_logger.statusMessage(statusMessagePrefix()
-            + "ERROR (See log for details)");
+            + Messages.getInstance().getString("SerializedModelSaver_AcceptClusterer_StatusMessage_Text_Second"));
       } else {
         System.err.println(message);
       }
@@ -358,16 +367,16 @@ public class SerializedModelSaver
 
       String prefix = "";
       try {
-        prefix = Environment.substitute(m_filenamePrefix);
+        prefix = m_env.substitute(m_filenamePrefix);
       } catch (Exception ex) {
         stop(); // stop processing
-        String message = "[SerializedModelSaver] "
-          + statusMessagePrefix() + " Can't save model. Reason: " 
+        String message = Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_First")
+          + statusMessagePrefix() + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_Second") 
           + ex.getMessage();
         if (m_logger != null) {
           m_logger.logMessage(message);
           m_logger.statusMessage(statusMessagePrefix()
-              + "ERROR (See log for details)");
+              + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_StatusMessage_Text_First"));
         } else {
           System.err.println(message);
         }
@@ -379,16 +388,16 @@ public class SerializedModelSaver
 
       String dirName = m_directory.getPath();
       try {
-        dirName = Environment.substitute(dirName);
+        dirName = m_env.substitute(dirName);
       } catch (Exception ex) {
         stop(); // stop processing
-        String message = "[SerializedModelSaver] "
-          + statusMessagePrefix() + " Can't save model. Reason: " 
+        String message = Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_Third")
+          + statusMessagePrefix() + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_Fourth") 
           + ex.getMessage();
         if (m_logger != null) {
           m_logger.logMessage(message);
           m_logger.statusMessage(statusMessagePrefix()
-              + "ERROR (See log for details)");
+              + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_StatusMessage_Text_Second"));
         } else {
           System.err.println(message);
         }
@@ -422,16 +431,16 @@ public class SerializedModelSaver
 
     String prefix = "";
     try {
-      prefix = Environment.substitute(m_filenamePrefix);
+      prefix = m_env.substitute(m_filenamePrefix);
     } catch (Exception ex) {
       stop(); // stop processing
-      String message = "[SerializedModelSaver] "
-        + statusMessagePrefix() + " Can't save model. Reason: " 
+      String message = Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_Fifth")
+        + statusMessagePrefix() + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_Sixth") 
         + ex.getMessage();
       if (m_logger != null) {
         m_logger.logMessage(message);
         m_logger.statusMessage(statusMessagePrefix()
-            + "ERROR (See log for details)");
+            + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_StatusMessage_Text_Third"));
       } else {
         System.err.println(message);
       }
@@ -448,16 +457,16 @@ public class SerializedModelSaver
     
     String dirName = m_directory.getPath();
     try {
-      dirName = Environment.substitute(dirName);
+      dirName = m_env.substitute(dirName);
     } catch (Exception ex) {
       stop(); // stop processing
-      String message = "[SerializedModelSaver] "
-        + statusMessagePrefix() + " Can't save model. Reason: " 
+      String message = Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_Seventh")
+        + statusMessagePrefix() + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_Message_Text_Eighth") 
                            + ex.getMessage();
       if (m_logger != null) {
         m_logger.logMessage(message);
         m_logger.statusMessage(statusMessagePrefix()
-            + "ERROR (See log for details)");
+            + Messages.getInstance().getString("SerializedModelSaver_AcceptClassifier_StatusMessage_Text_Fourth"));
       } else {
         System.err.println(message);
       }
@@ -475,6 +484,8 @@ public class SerializedModelSaver
   /**
    * Helper routine to actually save the models.
    */
+  
+  
   private void saveModel(String fileName, Instances trainHeader, Object model) {
     m_fileFormat = validateFileFormat(m_fileFormat);
     if (m_fileFormat == null) {
@@ -498,12 +509,12 @@ public class SerializedModelSaver
       }        
     } catch (Exception ex) {
       stop(); // stop all processing
-      System.err.println("[SerializedModelSaver] Problem saving model");
+      System.err.println(Messages.getInstance().getString("SerializedModelSaver_SaveModel_Error_Text"));
       if (m_logger != null) {
-        m_logger.logMessage("[SerializedModelSaver] "
-            + statusMessagePrefix() + " Problem saving model");
+        m_logger.logMessage(Messages.getInstance().getString("SerializedModelSaver_SaveModel_LogMessage_Text_First")
+            + statusMessagePrefix() + Messages.getInstance().getString("SerializedModelSaver_SaveModel_LogMessage_Text_Second"));
         m_logger.statusMessage(statusMessagePrefix()
-            + "ERROR (See log for details)");
+            + Messages.getInstance().getString("SerializedModelSaver_SaveModel_StatusMessage_Text"));
       }
     }
   }
@@ -629,7 +640,7 @@ public class SerializedModelSaver
    * @return information about this bean.
    */
   public String globalInfo() {
-    return "Save trained models to serialized object files.";
+    return Messages.getInstance().getString("SerializedModelSaver_GlobalInfo_Text");
   }
 
   /**
@@ -675,5 +686,25 @@ public class SerializedModelSaver
   
   private String statusMessagePrefix() {
     return getCustomName() + "$" + hashCode() + "|";
-  }   
+  }
+  
+  /**
+   * Set environment variables to use.
+   * 
+   * @param env the environment variables to
+   * use
+   */
+  public void setEnvironment(Environment env) {
+    m_env = env;
+  }
+  
+  // Custom de-serialization in order to set default
+  // environment variables on de-serialization
+  private void readObject(ObjectInputStream aStream) 
+    throws IOException, ClassNotFoundException {
+    aStream.defaultReadObject();
+    
+    // set a default environment to use
+    m_env = Environment.getSystemWide();
+  }
 }
\ No newline at end of file
