@@ -23,6 +23,7 @@
 package weka.gui.explorer;
 
 import weka.core.Capabilities;
+import weka.core.CapabilitiesHandler;
 import weka.core.Instances;
 import weka.core.OptionHandler;
 import weka.core.Utils;
@@ -67,6 +68,7 @@ import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
+import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.io.BufferedOutputStream;
@@ -102,7 +104,7 @@ import javax.swing.filechooser.FileFilter;
  *
  * @author Richard Kirkby (rkirkby@cs.waikato.ac.nz)
  * @author Len Trigg (trigg@cs.waikato.ac.nz)
- * @version $Revision: 1.73 $
+ * @version $Revision: 7529 $
  */
 public class PreprocessPanel
   extends JPanel 
@@ -116,31 +118,31 @@ public class PreprocessPanel
     new InstancesSummaryPanel();
 
   /** Click to load base instances from a file */
-  protected JButton m_OpenFileBut = new JButton("Open file...");
+  protected JButton m_OpenFileBut = new JButton(Messages.getInstance().getString("PreprocessPanel_OpenFileBut_JButton_Text"));
 
   /** Click to load base instances from a URL */
-  protected JButton m_OpenURLBut = new JButton("Open URL...");
+  protected JButton m_OpenURLBut = new JButton(Messages.getInstance().getString("PreprocessPanel_OpenURLBut_JButton_Text"));
 
   /** Click to load base instances from a Database */
-  protected JButton m_OpenDBBut = new JButton("Open DB...");
+  protected JButton m_OpenDBBut = new JButton(Messages.getInstance().getString("PreprocessPanel_OpenDBBut_JButton_Text"));
 
   /** Click to generate artificial data */
-  protected JButton m_GenerateBut = new JButton("Generate...");
+  protected JButton m_GenerateBut = new JButton(Messages.getInstance().getString("PreprocessPanel_GenerateBut_JButton_Text"));
 
   /** Click to revert back to the last saved point */
-  protected JButton m_UndoBut = new JButton("Undo");
+  protected JButton m_UndoBut = new JButton(Messages.getInstance().getString("PreprocessPanel_UndoBut_JButton_Text"));
 
   /** Click to open the current instances in a viewer */
-  protected JButton m_EditBut = new JButton("Edit...");
+  protected JButton m_EditBut = new JButton(Messages.getInstance().getString("PreprocessPanel_EditBut_JButton_Text"));
 
   /** Click to apply filters and save the results */
-  protected JButton m_SaveBut = new JButton("Save...");
+  protected JButton m_SaveBut = new JButton(Messages.getInstance().getString("PreprocessPanel_SaveBut_JButton_Text"));
   
   /** Panel to let the user toggle attributes */
   protected AttributeSelectionPanel m_AttPanel = new AttributeSelectionPanel();
 
   /** Button for removing attributes */
-  protected JButton m_RemoveButton = new JButton("Remove");
+  protected JButton m_RemoveButton = new JButton(Messages.getInstance().getString("PreprocessPanel_RemoveButton_JButton_Text"));
 
   /** Displays summary stats on the selected attribute */
   protected AttributeSummaryPanel m_AttSummaryPanel =
@@ -154,11 +156,11 @@ public class PreprocessPanel
   protected PropertyPanel m_FilterPanel = new PropertyPanel(m_FilterEditor);
 
   /** Click to apply filters and save the results */
-  protected JButton m_ApplyFilterBut = new JButton("Apply");
+  protected JButton m_ApplyFilterBut = new JButton(Messages.getInstance().getString("PreprocessPanel_ApplyFilterBut_JButton_Text"));
 
-  /** The file chooser for selecting arff files */
+  /** The file chooser for selecting data files */
   protected ConverterFileChooser m_FileChooser 
-    = new ConverterFileChooser(new File(System.getProperty("user.dir")));
+    = new ConverterFileChooser(new File(ExplorerDefaults.getInitialDirectory()));
 
   /** Stores the last URL that instances were loaded from */
   protected String m_LastURL = "http://";
@@ -210,16 +212,39 @@ public class PreprocessPanel
     m_FilterEditor.setClassType(weka.filters.Filter.class);
     if (ExplorerDefaults.getFilter() != null)
       m_FilterEditor.setValue(ExplorerDefaults.getFilter());
-    m_OpenFileBut.setToolTipText("Open a set of instances from a file");
-    m_OpenURLBut.setToolTipText("Open a set of instances from a URL");
-    m_OpenDBBut.setToolTipText("Open a set of instances from a database");
-    m_GenerateBut.setToolTipText("Generates artificial data");
-    m_UndoBut.setToolTipText("Undo the last change to the dataset");
-    m_EditBut.setToolTipText("Open the current dataset in a Viewer for editing");
-    m_SaveBut.setToolTipText("Save the working relation to a file");
-    m_ApplyFilterBut.setToolTipText("Apply the current filter to the data");
+    
+    m_FilterEditor.addPropertyChangeListener(new PropertyChangeListener() {
+      public void propertyChange(PropertyChangeEvent e) {
+        m_ApplyFilterBut.setEnabled(getInstances() != null);
+        Capabilities currentCapabilitiesFilter = m_FilterEditor.getCapabilitiesFilter();
+        Filter filter = (Filter) m_FilterEditor.getValue();
+        Capabilities currentFilterCapabilities = null;
+        if (filter != null && currentCapabilitiesFilter != null &&
+            (filter instanceof CapabilitiesHandler)) {
+          currentFilterCapabilities = ((CapabilitiesHandler)filter).getCapabilities();
+          
+          if (!currentFilterCapabilities.supportsMaybe(currentCapabilitiesFilter) &&
+              !currentFilterCapabilities.supports(currentCapabilitiesFilter)) {
+            m_ApplyFilterBut.setEnabled(false);
+          }
+        }
+      }
+    });
+    m_OpenFileBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_OpenFileBut_SetToolTipText_Text"));
+    m_OpenURLBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_OpenURLBut_SetToolTipText_Text"));
+    m_OpenDBBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_OpenDBBut_SetToolTipText_Text"));
+    m_GenerateBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_GenerateBut_SetToolTipText_Text"));
+    m_UndoBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_UndoBut_SetToolTipText_Text"));
+    m_UndoBut.setEnabled(ExplorerDefaults.get("enableUndo", "true").equalsIgnoreCase("true"));
+    if (!m_UndoBut.isEnabled()) {
+      m_UndoBut.setToolTipText("Undo is disabled - " +
+                "see weka.gui.explorer.Explorer.props to enable");
+    }
+    m_EditBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_EditBut_SetToolTipText_Text"));
+    m_SaveBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_SaveBut_SetToolTipText_Text"));
+    m_ApplyFilterBut.setToolTipText(Messages.getInstance().getString("PreprocessPanel_ApplyFilterBut_SetToolTipText_Text"));
 
-    m_FileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
+    m_FileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
     m_OpenURLBut.addActionListener(new ActionListener() {
       public void actionPerformed(ActionEvent e) {
 	setInstancesFromURLQ();
@@ -281,15 +306,14 @@ public class PreprocessPanel
     });
 
 
-    m_InstSummaryPanel.setBorder(BorderFactory
-				 .createTitledBorder("Current relation"));
+    m_InstSummaryPanel.setBorder(BorderFactory.createTitledBorder(Messages.getInstance().getString("PreprocessPanel_InstSummaryPanel_BorderFactoryCreateTitledBorder_Text")));
     JPanel attStuffHolderPanel = new JPanel();
     attStuffHolderPanel.setBorder(BorderFactory
-				  .createTitledBorder("Attributes"));
+				  .createTitledBorder(Messages.getInstance().getString("PreprocessPanel_AttStuffHolderPanel_BorderFactoryCreateTitledBorder_Text")));
     attStuffHolderPanel.setLayout(new BorderLayout());
     attStuffHolderPanel.add(m_AttPanel, BorderLayout.CENTER);
     m_RemoveButton.setEnabled(false);
-    m_RemoveButton.setToolTipText("Remove selected attributes.");
+    m_RemoveButton.setToolTipText(Messages.getInstance().getString("PreprocessPanel_RemoveButton_SetToolTipText_Text"));
     m_RemoveButton.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent e) {
 	  try {
@@ -301,11 +325,11 @@ public class PreprocessPanel
 	    if (selected.length == m_Instances.numAttributes()) {
 	      // Pop up an error optionpane
 	      JOptionPane.showMessageDialog(PreprocessPanel.this,
-					    "Can't remove all attributes from data!\n",
-					    "Remove Attributes",
+	    		  Messages.getInstance().getString("PreprocessPanel_JOptionPaneShowMessageDialog_Text_First"),
+	    		  Messages.getInstance().getString("PreprocessPanel_JOptionPaneShowMessageDialog_Text_Second"),
 					    JOptionPane.ERROR_MESSAGE);
-	      m_Log.logMessage("Can't remove all attributes from data!");
-	      m_Log.statusMessage("Problem removing attributes");
+	      m_Log.logMessage(Messages.getInstance().getString("PreprocessPanel_Log_LogMessage_Text_First"));
+	      m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_Log_StatusMessage_Text_First"));
 	      return;
 	    }
 	    r.setAttributeIndicesArray(selected);
@@ -316,12 +340,12 @@ public class PreprocessPanel
 	    }
 	    // Pop up an error optionpane
 	    JOptionPane.showMessageDialog(PreprocessPanel.this,
-					  "Problem filtering instances:\n"
+	    		Messages.getInstance().getString("PreprocessPanel_JOptionPaneShowMessageDialog_Text_Third")
 					  + ex.getMessage(),
-					  "Remove Attributes",
+					  Messages.getInstance().getString("PreprocessPanel_JOptionPaneShowMessageDialog_Text_Fourth"),
 					  JOptionPane.ERROR_MESSAGE);
-	    m_Log.logMessage("Problem removing attributes: " + ex.getMessage());
-	    m_Log.statusMessage("Problem removing attributes");
+	    m_Log.logMessage(Messages.getInstance().getString("PreprocessPanel_Log_LogMessage_Text_Second") + ex.getMessage());
+	    m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_Log_StatusMessage_Text_Second"));
 	  }
 	}
       });
@@ -332,7 +356,7 @@ public class PreprocessPanel
     p1.add(m_RemoveButton, BorderLayout.CENTER);
     attStuffHolderPanel.add(p1, BorderLayout.SOUTH);
     m_AttSummaryPanel.setBorder(BorderFactory
-		    .createTitledBorder("Selected attribute"));
+		    .createTitledBorder(Messages.getInstance().getString("PreprocessPanel_AttSummaryPanel_BorderFactoryCreateTitledBorder_Text")));
     m_UndoBut.setEnabled(false);
     m_EditBut.setEnabled(false);
     m_SaveBut.setEnabled(false);
@@ -357,7 +381,7 @@ public class PreprocessPanel
 
     JPanel filter = new JPanel();
     filter.setBorder(BorderFactory
-		    .createTitledBorder("Filter"));
+		    .createTitledBorder(Messages.getInstance().getString("PreprocessPanel_Filter_BorderFactoryCreateTitledBorder_Text")));
     filter.setLayout(new BorderLayout());
     filter.add(m_FilterPanel, BorderLayout.CENTER);
     filter.add(m_ApplyFilterBut, BorderLayout.EAST); 
@@ -367,8 +391,7 @@ public class PreprocessPanel
     attVis.add(m_AttSummaryPanel);
 
     JComboBox colorBox = m_AttVisualizePanel.getColorBox();
-    colorBox.setToolTipText("The chosen attribute will also be used as the " +
-			    "class attribute when a filter is applied.");
+    colorBox.setToolTipText(Messages.getInstance().getString("PreprocessPanel_ColorBox_SetToolTipText_Text"));
     colorBox.addItemListener(new ItemListener() {
       public void itemStateChanged(ItemEvent ie) {
 	if (ie.getStateChange() == ItemEvent.SELECTED) {
@@ -376,7 +399,7 @@ public class PreprocessPanel
 	}
       }
     });
-    final JButton visAllBut = new JButton("Visualize All");
+    final JButton visAllBut = new JButton(Messages.getInstance().getString("PreprocessPanel_VisAllBut_JButton_Text"));
     visAllBut.addActionListener(new ActionListener() {
 	public void actionPerformed(ActionEvent ae) {
 	  if (m_Instances != null) {
@@ -472,13 +495,15 @@ public class PreprocessPanel
 
 	  m_ApplyFilterBut.setEnabled(true);
 
-	  m_Log.logMessage("Base relation is now "
+	  /**
+	  m_Log.logMessage(Messages.getInstance().getString("PreprocessPanel_SetInstances_Run_Log_LogMessage_Text_First")
 			   + m_Instances.relationName()
-			   + " (" + m_Instances.numInstances()
-			   + " instances)");
+			   + Messages.getInstance().getString("PreprocessPanel_SetInstances_Run_Log_LogMessage_Text_Second") + m_Instances.numInstances()
+			   + Messages.getInstance().getString("PreprocessPanel_SetInstances_Run_Log_LogMessage_Text_Third"));
+	  **/
 	  m_SaveBut.setEnabled(true);
 	  m_EditBut.setEnabled(true);
-	  m_Log.statusMessage("OK");
+	  m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SetInstances_Run_Log_StatusMessage_Text_First"));
 	  // Fire a propertychange event
 	  m_Support.firePropertyChange("", null, null);
 	  
@@ -513,11 +538,13 @@ public class PreprocessPanel
       }
     } catch (Exception ex) {
       ex.printStackTrace();
+      /**
       JOptionPane.showMessageDialog(this,
-				    "Problem setting base instances:\n"
+    		  Messages.getInstance().getString("PreprocessPanel_SetInstances_Run_JOptionPaneShowMessageDialog_Text_First")
 				    + ex,
-				    "Instances",
+				    Messages.getInstance().getString("PreprocessPanel_SetInstances_Run_JOptionPaneShowMessageDialog_Text_Second"),
 				    JOptionPane.ERROR_MESSAGE);
+	  **/
     }
   }
 
@@ -568,26 +595,24 @@ public class PreprocessPanel
 	      if (m_Log instanceof TaskLogger) {
 		((TaskLogger)m_Log).taskStarted();
 	      }
-	      m_Log.statusMessage("Passing dataset through filter "
+	      m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_Log_StatusMessage_Text_First")
 		  + filter.getClass().getName());
 	      String cmd = filter.getClass().getName();
 	      if (filter instanceof OptionHandler)
 		cmd += " " + Utils.joinOptions(((OptionHandler) filter).getOptions());
-	      m_Log.logMessage("Command: " + cmd);
+	      m_Log.logMessage(Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_Log_LogMessage_Text_First") + cmd);
 	      int classIndex = m_AttVisualizePanel.getColoringIndex();
 	      if ((classIndex < 0) && (filter instanceof SupervisedFilter)) {
-		throw new IllegalArgumentException("Class (colour) needs to " +
-						   "be set for supervised " +
-						   "filter.");
+		throw new IllegalArgumentException(Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_IllegalArgumentException_Text"));
 	      }
 	      Instances copy = new Instances(m_Instances);
 	      copy.setClassIndex(classIndex);
 	      filter.setInputFormat(copy);
 	      Instances newInstances = Filter.useFilter(copy, filter);
 	      if (newInstances == null || newInstances.numAttributes() < 1) {
-		throw new Exception("Dataset is empty.");
+		throw new Exception(Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_Exception_Text"));
 	      }
-	      m_Log.statusMessage("Saving undo information");
+	      m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_Log_StatusMessage_Text_Second"));
 	      addUndoPoint();
 	      m_AttVisualizePanel.setColoringIndex(copy.classIndex());
 	      // if class was not set before, reset it again after use of filter
@@ -607,12 +632,12 @@ public class PreprocessPanel
 	    }
 	    // Pop up an error optionpane
 	    JOptionPane.showMessageDialog(PreprocessPanel.this,
-					  "Problem filtering instances:\n"
+	    		Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_JOptionPaneShowMessageDialog_Text_First")
 					  + ex.getMessage(),
-					  "Apply Filter",
+					  Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_JOptionPaneShowMessageDialog_Text_Second"),
 					  JOptionPane.ERROR_MESSAGE);
-	    m_Log.logMessage("Problem filtering instances: " + ex.getMessage());
-	    m_Log.statusMessage("Problem filtering instances");
+	    m_Log.logMessage(Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_Log_LogMessage_Text_Second") + ex.getMessage());
+	    m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_Log_StatusMessage_Text_Third"));
 	  }
 	  m_IOThread = null;
 	}
@@ -621,9 +646,8 @@ public class PreprocessPanel
       m_IOThread.start();
     } else {
       JOptionPane.showMessageDialog(this,
-				    "Can't apply filter at this time,\n"
-				    + "currently busy with other IO",
-				    "Apply Filter",
+    		  Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_JOptionPaneShowMessageDialog_Text_Third"),
+				    Messages.getInstance().getString("PreprocessPanel_ApplyFilter_Run_JOptionPaneShowMessageDialog_Text_Fourth"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -650,9 +674,8 @@ public class PreprocessPanel
     }
     else {
       JOptionPane.showMessageDialog(this,
-				    "Can't save at this time,\n"
-				    + "currently busy with other IO",
-				    "Save Instances",
+    		  Messages.getInstance().getString("PreprocessPanel_SaveWorkingInstancesToFileQ_JOptionPaneShowMessageDialog_Text_First"),
+				    Messages.getInstance().getString("PreprocessPanel_SaveWorkingInstancesToFileQ_JOptionPaneShowMessageDialog_Text_Second"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -668,12 +691,12 @@ public class PreprocessPanel
       m_IOThread = new Thread() {
 	  public void run() {
 	    try {
-	      m_Log.statusMessage("Saving to file...");
+	      m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SaveInstancesToFile_Run_Log_StatusMessage_Text_First"));
 
 	      saver.setInstances(inst);
 	      saver.writeBatch();
 	      
-	      m_Log.statusMessage("OK");
+	      m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SaveInstancesToFile_Run_Log_StatusMessage_Text_Second"));
 	    }
 	    catch (Exception ex) {
 	      ex.printStackTrace();
@@ -687,9 +710,8 @@ public class PreprocessPanel
     }
     else {
       JOptionPane.showMessageDialog(this,
-				    "Can't save at this time,\n"
-				    + "currently busy with other IO",
-				    "Saving instances",
+    		  Messages.getInstance().getString("PreprocessPanel_SaveInstancesToFile_Run_JOptionPaneShowMessageDialog_Text_First"),
+				    Messages.getInstance().getString("PreprocessPanel_SaveInstancesToFile_Run_JOptionPaneShowMessageDialog_Text_Second"),
 				    JOptionPane.WARNING_MESSAGE);
     } 
   }
@@ -713,8 +735,8 @@ public class PreprocessPanel
 
 	if (m_FileChooser.getLoader() == null) {
 	  JOptionPane.showMessageDialog(this,
-	      "Cannot determine file loader automatically, please choose one.",
-	      "Load Instances",
+			  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFileQ_JOptionPaneShowMessageDialog_Text_First"),
+			  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFileQ_JOptionPaneShowMessageDialog_Text_Second"),
 	      JOptionPane.ERROR_MESSAGE);
 	  converterQuery(m_FileChooser.getSelectedFile());
 	}
@@ -725,9 +747,8 @@ public class PreprocessPanel
       }
     } else {
       JOptionPane.showMessageDialog(this,
-				    "Can't load at this time,\n"
-				    + "currently busy with other IO",
-				    "Load Instances",
+    		  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFileQ_JOptionPaneShowMessageDialog_Text_Third"),
+				    Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFileQ_JOptionPaneShowMessageDialog_Text_Fourth"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -763,17 +784,16 @@ public class PreprocessPanel
 	setInstancesFromDB(InstQ);
       } catch (Exception ex) {
 	JOptionPane.showMessageDialog(this,
-				      "Problem connecting to database:\n"
+			Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDBQ_JOptionPaneShowMessageDialog_Text_First")
 				      + ex.getMessage(),
-				      "Load Instances",
+				      Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDBQ_JOptionPaneShowMessageDialog_Text_Second"),
 				      JOptionPane.ERROR_MESSAGE);
       }
       
     } else {
       JOptionPane.showMessageDialog(this,
-				     "Can't load at this time,\n"
-				    + "currently busy with other IO",
-				    "Load Instances",
+    		  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDBQ_JOptionPaneShowMessageDialog_Text_Third"),
+				    Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDBQ_JOptionPaneShowMessageDialog_Text_Fourth"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -788,8 +808,8 @@ public class PreprocessPanel
     if (m_IOThread == null) {
       try {
 	String urlName = (String) JOptionPane.showInputDialog(this,
-			"Enter the source URL",
-			"Load Instances",
+			Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURLQ_JOptionPaneShowMessageDialog_Text_First"),
+			Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURLQ_JOptionPaneShowMessageDialog_Text_Second"),
 			JOptionPane.QUESTION_MESSAGE,
 			null,
 			null,
@@ -805,16 +825,15 @@ public class PreprocessPanel
       } catch (Exception ex) {
 	ex.printStackTrace();
 	JOptionPane.showMessageDialog(this,
-				      "Problem with URL:\n"
+			Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURLQ_JOptionPaneShowMessageDialog_Text_Third")
 				      + ex.getMessage(),
-				      "Load Instances",
+				      Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURLQ_JOptionPaneShowMessageDialog_Text_Fourth"),
 				      JOptionPane.ERROR_MESSAGE);
       }
     } else {
       JOptionPane.showMessageDialog(this,
-				    "Can't load at this time,\n"
-				    + "currently busy with other IO",
-				    "Load Instances",
+    		  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURLQ_JOptionPaneShowMessageDialog_Text_Fifth"),
+				    Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURLQ_JOptionPaneShowMessageDialog_Text_Sixth"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -830,9 +849,9 @@ public class PreprocessPanel
               // create dialog
               final DataGeneratorPanel generatorPanel = new DataGeneratorPanel();
               final JDialog dialog = new JDialog();
-              final JButton generateButton = new JButton("Generate");
+              final JButton generateButton = new JButton(Messages.getInstance().getString("PreprocessPanel_GenerateInstances_Run_GenerateButton_JButton_Text"));
               final JCheckBox showOutputCheckBox = 
-                                  new JCheckBox("Show generated data as text, incl. comments");
+                                  new JCheckBox(Messages.getInstance().getString("PreprocessPanel_GenerateInstances_Run_ShowOutputCheckBox_JCheckBox_Text"));
 
               showOutputCheckBox.setMnemonic('S');
               generatorPanel.setLog(m_Log);
@@ -842,7 +861,7 @@ public class PreprocessPanel
                         300, 
                         (int) generatorPanel.getPreferredSize().getHeight()));
               generateButton.setMnemonic('G');
-              generateButton.setToolTipText("Generates the dataset according the settings.");
+              generateButton.setToolTipText(Messages.getInstance().getString("PreprocessPanel_GenerateInstances_Run_GenerateButton_SetToolTipText_Text"));
               generateButton.addActionListener(new ActionListener(){
                   public void actionPerformed(ActionEvent evt){
                     // generate
@@ -862,7 +881,7 @@ public class PreprocessPanel
                       showGeneratedInstances(generatorPanel.getOutput());
                 }
               });
-              dialog.setTitle("DataGenerator");
+              dialog.setTitle(Messages.getInstance().getString("PreprocessPanel_GenerateInstances_Run_Dialog_Text"));
               dialog.getContentPane().add(generatorPanel, BorderLayout.CENTER);
               dialog.getContentPane().add(generateButton, BorderLayout.EAST);
               dialog.getContentPane().add(showOutputCheckBox, BorderLayout.SOUTH);
@@ -883,9 +902,8 @@ public class PreprocessPanel
     } 
     else {
       JOptionPane.showMessageDialog(this,
-				    "Can't generate data at this time,\n"
-				    + "currently busy with other IO",
-				    "Generate Data",
+    		  Messages.getInstance().getString("PreprocessPanel_GenerateInstances_Run_JOptionPaneShowMessageDialog_Text_First"),
+				    Messages.getInstance().getString("PreprocessPanel_GenerateInstances_Run_JOptionPaneShowMessageDialog_Text_Second"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -897,8 +915,8 @@ public class PreprocessPanel
    */
   protected void showGeneratedInstances(String data) {
     final JDialog dialog = new JDialog();
-    final JButton saveButton = new JButton("Save");
-    final JButton closeButton = new JButton("Close");
+    final JButton saveButton = new JButton(Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_SaveButton_JButton_Text"));
+    final JButton closeButton = new JButton(Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_CloseButton_JButton_Text"));
     final JTextArea textData = new JTextArea(data);
     final JPanel panel = new JPanel();
     panel.setLayout(new FlowLayout(FlowLayout.RIGHT));
@@ -907,7 +925,7 @@ public class PreprocessPanel
         new Font("Monospaced", Font.PLAIN, textData.getFont().getSize()));
 
     saveButton.setMnemonic('S');
-    saveButton.setToolTipText("Saves the output to a file");
+    saveButton.setToolTipText(Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_SaveButton_SetToolTipText_Text"));
     saveButton.addActionListener(new ActionListener(){
       public void actionPerformed(ActionEvent evt){
         JFileChooser filechooser = new JFileChooser();
@@ -922,9 +940,9 @@ public class PreprocessPanel
             writer.close();
             JOptionPane.showMessageDialog(
               dialog, 
-              "Output successfully saved to file '" 
-              + filechooser.getSelectedFile() + "'!",
-              "Information",
+              Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_SaveButton_JOptionPaneShowMessageDialog_Text_First") 
+              + filechooser.getSelectedFile() + Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_SaveButton_JOptionPaneShowMessageDialog_Text_Second"),
+              Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_SaveButton_JOptionPaneShowMessageDialog_Text_Third"),
               JOptionPane.INFORMATION_MESSAGE);
           }
           catch (Exception e) {
@@ -935,7 +953,7 @@ public class PreprocessPanel
       }
     });
     closeButton.setMnemonic('C');
-    closeButton.setToolTipText("Closes the dialog");
+    closeButton.setToolTipText(Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_CloseButton_SetToolTipText_Text"));
     closeButton.addActionListener(new ActionListener(){
       public void actionPerformed(ActionEvent evt){
         dialog.dispose();
@@ -943,7 +961,7 @@ public class PreprocessPanel
     });
     panel.add(saveButton);
     panel.add(closeButton);
-    dialog.setTitle("Generated Instances (incl. comments)");
+    dialog.setTitle(Messages.getInstance().getString("PreprocessPanel_ShowGeneratedInstances_Dialog_SetToolTipText_Text"));
     dialog.getContentPane().add(new JScrollPane(textData), BorderLayout.CENTER);
     dialog.getContentPane().add(panel, BorderLayout.SOUTH);
     dialog.pack();
@@ -1004,13 +1022,13 @@ public class PreprocessPanel
 	      Instances inst = cnv.getDataSet();
 	      setInstances(inst);
 	    } catch (Exception ex) {
-	      m_Log.statusMessage(cnv.getClass().getName()+" failed to load "
+	      m_Log.statusMessage(cnv.getClass().getName()+Messages.getInstance().getString("PreprocessPanel_TryConverter_Log_StatusMessage_Text")
 				 +f.getName());
 	      JOptionPane.showMessageDialog(PreprocessPanel.this,
-					    cnv.getClass().getName()+" failed to load '"
-					    + f.getName() + "'.\n"
-					    + "Reason:\n" + ex.getMessage(),
-					    "Convert File",
+					    cnv.getClass().getName()+Messages.getInstance().getString("PreprocessPanel_TryConverter_JOptionPaneShowMessageDialog_Text_First")
+					    + f.getName() + Messages.getInstance().getString("PreprocessPanel_TryConverter_JOptionPaneShowMessageDialog_Text_Second")
+					    + Messages.getInstance().getString("PreprocessPanel_TryConverter_JOptionPaneShowMessageDialog_Text_Third") + ex.getMessage(),
+					    Messages.getInstance().getString("PreprocessPanel_TryConverter_JOptionPaneShowMessageDialog_Text_Fourth"),
 					    JOptionPane.ERROR_MESSAGE);
 	      m_IOThread = null;
 	      converterQuery(f);
@@ -1036,26 +1054,26 @@ public class PreprocessPanel
       m_IOThread = new Thread() {
 	public void run() {
 	  try {
-	    m_Log.statusMessage("Reading from file...");
+	    m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_Log_StatusMessage_Text_First"));
 	    Instances inst = loader.getDataSet();
 	    setInstances(inst);
 	  }
 	  catch (Exception ex) {
 	    m_Log.statusMessage(
-		"File '" + loader.retrieveFile() + "' not recognised as an '"
-		+ loader.getFileDescription() + "' file.");
+	    		Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_Log_StatusMessage_Text_Second") + loader.retrieveFile() + Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_Log_StatusMessage_Text_Third")
+		+ loader.getFileDescription() + Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_Log_StatusMessage_Text_Fourth"));
 	    m_IOThread = null;
 	    if (JOptionPane.showOptionDialog(PreprocessPanel.this,
-					     "File '" + loader.retrieveFile()
-					     + "' not recognised as an '"
+	    		Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_First") + loader.retrieveFile()
+					     + Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Second")
 					     + loader.getFileDescription() 
-					     + "' file.\n"
-					     + "Reason:\n" + ex.getMessage(),
-					     "Load Instances",
+					     + Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Third")
+					     + Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Fourth") + ex.getMessage(),
+					     Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Fifth"),
 					     0,
 					     JOptionPane.ERROR_MESSAGE,
 					     null,
-					     new String[] {"OK", "Use Converter"},
+					     new String[] {Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Sixth"), Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Seventh")},
 					     null) == 1) {
 	    
 	      converterQuery(loader.retrieveFile());
@@ -1068,9 +1086,8 @@ public class PreprocessPanel
       m_IOThread.start();
     } else {
       JOptionPane.showMessageDialog(this,
-				    "Can't load at this time,\n"
-				    + "currently busy with other IO",
-				    "Load Instances",
+    		  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Eigth"),
+				    Messages.getInstance().getString("PreprocessPanel_SetInstancesFromFile_Run_JOptionPaneShowOptionDialog_Text_Nineth"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -1087,7 +1104,7 @@ public class PreprocessPanel
 	public void run() {
 	  
 	  try {
-	    m_Log.statusMessage("Reading from database...");
+	    m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_Log_StatusMessage_Text_First"));
 	    final Instances i = iq.retrieveInstances();
 	    SwingUtilities.invokeAndWait(new Runnable() {
 	      public void run() {
@@ -1096,11 +1113,11 @@ public class PreprocessPanel
 	    });
 	    iq.disconnectFromDatabase();
 	  } catch (Exception ex) {
-	    m_Log.statusMessage("Problem executing DB query "+m_SQLQ);
+	    m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_Log_StatusMessage_Text_Second") + m_SQLQ);
 	    JOptionPane.showMessageDialog(PreprocessPanel.this,
-					  "Couldn't read from database:\n"
+	    		Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_First")
 					  + ex.getMessage(),
-					  "Load Instances",
+					  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Second"),
 					  JOptionPane.ERROR_MESSAGE);
 	  }
 
@@ -1112,9 +1129,8 @@ public class PreprocessPanel
       m_IOThread.start();
     } else {
        JOptionPane.showMessageDialog(this,
-				    "Can't load at this time,\n"
-				    + "currently busy with other IO",
-				    "Load Instances",
+    		   Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Third"),
+				    Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Fourth"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -1131,20 +1147,20 @@ public class PreprocessPanel
 	public void run() {
 
 	  try {
-	    m_Log.statusMessage("Reading from URL...");
+	    m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURL_Run_Log_StatusMessage_Text_First"));
 	    AbstractFileLoader loader = ConverterUtils.getURLLoaderForFile(u.toString());
 	    if (loader == null)
-	      throw new Exception("No suitable URLSourcedLoader found for URL!\n" + u);
+	      throw new Exception(Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURL_Run_Exception_Text_First") + u);
 	    ((URLSourcedLoader) loader).setURL(u.toString());
 	    setInstances(loader.getDataSet());
 	  } catch (Exception ex) {
 	    ex.printStackTrace();
-	    m_Log.statusMessage("Problem reading " + u);
+	    m_Log.statusMessage(Messages.getInstance().getString("PreprocessPanel_SetInstancesFromURL_Run_Log_StatusMessage_Text_Second") + u);
 	    JOptionPane.showMessageDialog(PreprocessPanel.this,
-					  "Couldn't read from URL:\n"
-					  + u + "\n"
+	    		Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Fifth")
+					  + u + Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Sixth")
 					  + ex.getMessage(),
-					  "Load Instances",
+					  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Seventh"),
 					  JOptionPane.ERROR_MESSAGE);
 	  }
 
@@ -1155,9 +1171,8 @@ public class PreprocessPanel
       m_IOThread.start();
     } else {
       JOptionPane.showMessageDialog(this,
-				    "Can't load at this time,\n"
-				    + "currently busy with other IO",
-				    "Load Instances",
+    		  Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Eighth"),
+				    Messages.getInstance().getString("PreprocessPanel_SetInstancesFromDB_Run_JOptionPaneShowMessageDialog_Text_Nineth"),
 				    JOptionPane.WARNING_MESSAGE);
     }
   }
@@ -1168,11 +1183,34 @@ public class PreprocessPanel
    * @throws Exception 	if an error occurs
    */
   public void addUndoPoint() throws Exception {
+    if (!ExplorerDefaults.get("enableUndo", "true").equalsIgnoreCase("true")) {
+      return;
+    }
     
     if (m_Instances != null) {
       // create temporary file
       File tempFile = File.createTempFile("weka", SerializedInstancesLoader.FILE_EXTENSION);
       tempFile.deleteOnExit();
+      
+      if (!ExplorerDefaults.get("undoDirectory", "%t").equalsIgnoreCase("%t")) {
+        String dir = ExplorerDefaults.get("undoDirectory", "%t");
+        File undoDir = new File(dir);
+        if (undoDir.exists()) {
+          String fileName = tempFile.getName();
+          File newFile = new File(dir + File.separator + fileName);
+          if (undoDir.canWrite()) {
+            newFile.deleteOnExit();
+            tempFile = newFile;
+          } else {
+            System.err.println("Explorer: it doesn't look like we have permission" +
+                        " to write to the user-specified undo directory " +
+                        "'" + dir + "'");
+          }
+        } else {
+          System.err.println("Explorer: user-specified undo directory '" +
+              dir + "' does not exist!");
+        }
+      }
 
       ObjectOutputStream oos = 
 	new ObjectOutputStream(
@@ -1219,8 +1257,8 @@ public class PreprocessPanel
 	e.printStackTrace();
 	m_Log.logMessage(e.toString());
 	JOptionPane.showMessageDialog(PreprocessPanel.this,
-	    "Cannot perform undo operation!\n" + e.toString(),
-	    "Undo",
+			Messages.getInstance().getString("PreprocessPanel_Undo_JOptionPaneShowMessageDialog_Text_First") + e.toString(),
+			Messages.getInstance().getString("PreprocessPanel_Undo_JOptionPaneShowMessageDialog_Text_Second"),
 	    JOptionPane.ERROR_MESSAGE);
       }
 
@@ -1313,6 +1351,21 @@ public class PreprocessPanel
     
     // set new filter
     m_FilterEditor.setCapabilitiesFilter(filterClass);
+    
+    // check capabilities
+    m_ApplyFilterBut.setEnabled(true);
+    Capabilities currentCapabilitiesFilter = m_FilterEditor.getCapabilitiesFilter();
+    Filter currentFilter = (Filter) m_FilterEditor.getValue();
+    Capabilities currentFilterCapabilities = null;
+    if (currentFilter != null && currentCapabilitiesFilter != null &&
+        (currentFilter instanceof CapabilitiesHandler)) {
+      currentFilterCapabilities = ((CapabilitiesHandler)currentFilter).getCapabilities();
+      
+      if (!currentFilterCapabilities.supportsMaybe(currentCapabilitiesFilter) &&
+          !currentFilterCapabilities.supports(currentCapabilitiesFilter)) {
+        m_ApplyFilterBut.setEnabled(false);
+      }
+    }
   }
   
   /**
@@ -1333,7 +1386,7 @@ public class PreprocessPanel
    * @return 		the title of this tab
    */
   public String getTabTitle() {
-    return "Preprocess";
+    return Messages.getInstance().getString("PreprocessPanel_GetTitle_Text");
   }
   
   /**
@@ -1342,7 +1395,7 @@ public class PreprocessPanel
    * @return 		the tooltip of this tab
    */
   public String getTabTitleToolTip() {
-    return "Open/Edit/Save instances";
+    return Messages.getInstance().getString("PreprocessPanel_GetTabTitleToolTip_Text");
   }
   
   /**
@@ -1353,7 +1406,7 @@ public class PreprocessPanel
   public static void main(String [] args) {
 
     try {
-      final JFrame jf = new JFrame("Weka Explorer: Preprocess");
+      final JFrame jf = new JFrame(Messages.getInstance().getString("PreprocessPanel_Main_JFRame_Text"));
       jf.getContentPane().setLayout(new BorderLayout());
       final PreprocessPanel sp = new PreprocessPanel();
       jf.getContentPane().add(sp, BorderLayout.CENTER);
